import os
from contextlib import suppress

from ._lephare import flt, write_output_filter
from .runner import Runner

__all__ = [
    "Filter",
]

config_keys = {
    "verbose": "increase onscreen verbosity",
    "FILTER_REP": "path to repository where filter files are searched",
    "FILTER_LIST": "list of filter files, to be searched in FILTER_REP if relative",
    "TRANS_TYPE": "Transmission curve type: 0[def] for energy, 1 for photon nb",
    "FILTER_CALIB": "calibration system for the filter:\
    0[def]: fnu=cst 1: nu.fnu=cst 2: fnu=nu 3: fnu=Black Body @ T=10000K\
    4: for MIPS (leff with nu fnu=ctt and flux with BB @ 10000K",
    "FILTER_FILE": "output filter filename, will be saved in $LEPHAREWORK/filt/",
}


class Filter(Runner):
    """
    The specific arguments to the Filter class are

    verbose
                increase onscreen verbosity
    FILTER_REP
                path to repository where filter files are searched
    FILTER_LIST
                list of filter files, to be searched in FILTER_REP if relative
    TRANS_TYPE
                Transmission curve type: 0[def] for energy, 1 for photon nb
    FILTER_CALIB
                calibration system for the filter:
                  0[def]: fnu=cst
                  1: nu.fnu=cst
                  2: fnu=nu
                  3: fnu=Black Body @ T=10000K
                  4: for MIPS (leff with nu fnu=ctt and flux with BB @ 10000K
    FILTER_FILE
                output filter filename, will be saved in $LEPHAREWORK/filt/
    """

    def update_help(self):
        """Add specific help information"""
        doc = "Build the LePHARE internal representation of the set of filters to be used\n"
        with suppress(Exception):
            self.parser.usage = doc
        self.__doc__ = doc + "\n"  # + inspect.getdoc(Filter)

    def __init__(self, config_file=None, config_keymap=None, **kwargs):
        super().__init__(config_keys, config_file, config_keymap, **kwargs)

    def run(self, **kwargs):
        """Update keymap and verbosity based on call arguments.

        This is only when the code is called from python session.
        """

        super().run(**kwargs)

        keymap = self.keymap
        flt_rep = keymap["FILTER_REP"].split_string(os.path.join(os.environ["LEPHAREDIR"], "filt/"), 1)[0]
        flt_files = keymap["FILTER_LIST"].split_string("flt.pb", -99)
        ref_size = len(flt_files)
        # Transmission in energy or photons
        transtyp = (keymap["TRANS_TYPE"]).split_int("0", ref_size)
        # calibration depending on the instrument
        calibtyp = (keymap["FILTER_CALIB"]).split_int("0", ref_size)
        # Output file
        output_name = keymap["FILTER_FILE"].split_string("filters", 1)[0]
        filtfile = os.path.join(os.environ["LEPHAREWORK"], "filt", output_name + ".dat")
        filtdoc = os.path.join(os.environ["LEPHAREWORK"], "filt", output_name + ".doc")

        if self.verbose:
            print("#######################################")
            print("# Build the filter file with the following options: ")
            print(f"# Config file: {self.config}")
            print(f"# FILTER_REP: {flt_rep}")
            print(f"# FILTER_LIST: {flt_files}")
            print(f"# TRANS_TYPE: {transtyp}")
            print(f"# FILTER_CALIB: {calibtyp}")
            print(f"# FILTER_FILE: {filtfile}")
            print(f"# FILTER_FILE.doc: {filtdoc}")
            print("#######################################")

        vec_flt = []
        for k, (f, t, c) in enumerate(zip(flt_files, transtyp, calibtyp)):
            flt_file = f if os.path.isabs(f) else os.path.join(flt_rep, f)
            one_filt = flt(k, flt_file, t, c)
            vec_flt.append(one_filt)

        write_output_filter(filtfile, filtdoc, vec_flt)
        return vec_flt


def main():  # pragma no cover
    runner = Filter()
    _ = runner.run()
    runner.end()


if __name__ == "__main__":  # pragma no cover
    main()



import os

import numpy as np
from astropy.table import Column, Table

from ._lephare import PhotoZ, readOutKeywords
from ._utils import continueClass

__all__ = [
    "PhotoZ",
]


@continueClass
class PhotoZ:  # noqa: F811
    pdfdict = {
        "MASS": 0,
        "SFR": 1,
        "SSFR": 2,
        "LDUST": 3,
        "LIR": 4,
        "AGE": 5,
        "COL1": 6,
        "COL2": 7,
        "MREF": 8,
        "MIN_ZG": 9,
        "MIN_ZQ": 10,
        "BAY_ZG": 11,
        "BAY_ZQ": 12,
    }

    def build_output_tables(self, srclist, para_out=None, filename=None):
        # BUILD THE TABLE OF THE OUTPUT PARAMETERS
        d = np.loadtxt(os.path.join(os.environ["LEPHAREDIR"], "alloutputkeys.txt"), dtype="str")
        allkeys = {}
        for count, label in enumerate(d[:, 0]):
            allkeys[label] = (d[count, 1], d[count, 2])
        outkeys = readOutKeywords(self.outpara) if para_out is None else readOutKeywords(para_out)
        outputs = {}
        t = Table()
        for key in outkeys:
            typ, attr = allkeys[key]
            is_array = False
            if "[" in attr:
                tmp = attr.split("[")
                attr = tmp[0]
                try:
                    index = int(tmp[1][0])
                except ValueError:
                    index = tmp[1][1:-2]
                is_array = True
            outputs[key] = []
            for src in srclist:
                if is_array is False:
                    outputs[key].append(getattr(src, attr))
                else:
                    outputs[key].append(getattr(src, attr)[index])
            t.add_column(Column(name=key, dtype=typ, data=outputs[key]))
        # NOW THE PDFS, IN THE SAME TABLE FOR NOW
        for typ in self.pdftype:
            arr = []
            key = self.pdfdict[typ]
            for src in srclist:
                pdf = src.pdfmap[key]
                arr.append(pdf.vPDF)
            t.add_column(Column(name="PDF_" + typ + "()", dtype=float, data=arr))
            # use the last src object to get the x-axis values
            t.meta[typ] = " ".join(str(e) for e in pdf.xaxis)

        if filename is not None:
            self.save_table(t, filename)
        return t

    def save_table(self, table, filename, fmt="fits", overwrite=True):
        if fmt == "fits":
            table.write(filename, "fits", overwrite)



import datetime
import os

import yaml

import lephare as lp

__all__ = [
    "prepare",
    "overwrite_config",
    "read_yaml_config",
    "write_yaml_config",
    "write_para_config",
    "keymap_to_string_dict",
    "string_dict_to_keymap",
    "all_types_to_keymap",
]

_DEFAULT = object()


def prepare(config, star_config=_DEFAULT, gal_config=_DEFAULT, qso_config=_DEFAULT):
    """Run the prepare stages of LePHARE

    In order to run "zphota" we must create the filter files, run sedtolib to
    create the SED libraries, and finally run mag_gal, to create the
    magnitude libararies. We abstract these tasks into a single prepare stage.

    We overide the config for each type if distinct config set. If no overide
    configs are set we use the same for each type.

    Parameters
    ==========
    config : dict of lephare.keyword
        The config base to run all tasks
    star_config : dict of lephare.keyword or None
        Config values to override for stars. If None do not run.
    gal_config : dict of lephare.keyword or None
        Config values to override for galaxies. If None do not run.
    qso_config : dict of lephare.keyword or None
        Config values to override for QSO. If None do not run.
    """
    # ensure that all values in the keymap are keyword objects
    config = all_types_to_keymap(config)
    object_types = dict(STAR=star_config, GAL=gal_config, QSO=qso_config)

    # Run the filter command
    # load filters from config
    filter_lib = lp.FilterSvc.from_keymap(config)
    # Get location to store filter files
    filter_output = os.path.join(os.environ["LEPHAREWORK"], "filt", config["FILTER_FILE"].value)
    # Write filter files
    lp.write_output_filter(filter_output + ".dat", filter_output + ".doc", filter_lib)
    # # Write config to work directory
    write_yaml_config(config, f"{filter_output}_config.yaml")

    for object_type in object_types:
        # Skip object types that are not to be processed
        if object_types[object_type] is None:
            continue
        elif object_types[object_type] is _DEFAULT:
            updated_config = config
        else:
            # Overwrite config for this object type
            object_types[object_type] = all_types_to_keymap(object_types[object_type])
            updated_config = overwrite_config(config, object_types[object_type])

        # Write the updated config
        sed_out_name = f"{updated_config[f'{object_type}_LIB'].value}_{object_type.lower()}_config.yaml"
        sed_output = os.path.join(os.environ["LEPHAREWORK"], "lib_bin", sed_out_name)
        mag_out_name = f"{updated_config[f'{object_type}_LIB_OUT'].value}_{object_type.lower()}_config.yaml"
        mag_output = os.path.join(os.environ["LEPHAREWORK"], "lib_mag", mag_out_name)
        # Run sedtolib
        sedlib = lp.Sedtolib(config_keymap=updated_config)
        list_loc = updated_config[f"{object_type}_SED"].value
        # if find sed/ in the path, assume it is in lephare-data and not an absolute path
        if list_loc.find("sed/") >= 0:
            list_loc = os.path.join(lp.LEPHAREDIR, list_loc)
        sedtolib_kwargs = {f"{object_type.lower()}_sed": list_loc}
        sedlib.run(typ=object_type, **sedtolib_kwargs)
        write_yaml_config(updated_config, sed_output)
        # Run mag_gal
        maglib = lp.MagGal(config_keymap=updated_config)
        maglib.run(typ=object_type, verbose=False)
        write_yaml_config(updated_config, mag_output)


def overwrite_config(config1, config2):
    """Check that two config can be safely broadcast for a joint run"""
    if config2 is None:
        return config1
    config = config1
    # Redshift grid must be idenitical
    # assert True
    # filters must be indentical
    # assert True
    for k in config2:
        config[k] = config2[k]
    return config


def read_yaml_config(yaml_file_path):
    """Open a standard yaml file and render it as a dictionary of keywords

    Parameters
    ==========
    yaml_file_path : str
        Path to input yaml file.
    """
    with open(yaml_file_path, "r") as file:
        config = yaml.safe_load(file)
    for key in config:
        config[key] = lp.keyword(key, config[key])
    return config


def write_yaml_config(keymap, yaml_file_path):
    """Write a dictionary of keywords to a yaml file

    Parameters
    ==========
    keymap : dict of lephare.keyword
        The dictionary of keywords to be written to yaml.
    yaml_file_path : str
        Path to output yaml file.
    """
    keymap = lp.all_types_to_keymap(keymap)
    config_dict = {}
    for k in keymap:
        config_dict[keymap[k].name] = keymap[k].value
    with open(yaml_file_path, "w") as yaml_file:
        now = datetime.datetime.now().strftime("%Y%m%dT%H%M%S")
        yaml_file.write(f"# File written automatically at {now} by lephare version {lp.__version__}\n")
        yaml.dump(config_dict, yaml_file)


def write_para_config(keymap, para_file_path):
    """Write a dictionary of keywords to a para file

    Parameters
    ==========
    keymap : dict of lephare.keyword
        The dictionary of keywords to be written to yaml.
    para_file_path : str
        Path to output para file.
    """
    keymap = lp.all_types_to_keymap(keymap)
    now = datetime.datetime.now().strftime("%Y%m%dT%H%M%S")
    para_contents = f"# File written automatically at {now} by lephare version {lp.__version__}\n"
    for k in keymap:
        para_contents += f"{k} {keymap[k].value}\n"
    with open(para_file_path, "w") as file_handle:
        file_handle.write(para_contents)
        file_handle.close()


def keymap_to_string_dict(keymap):
    """Convert a dictionary of keywords to a dictionary of strings"""
    config_dict = {}
    for k in keymap:
        config_dict[keymap[k].name] = keymap[k].value
    return config_dict


def string_dict_to_keymap(string_dict):
    """Convert a dictionary of strings to a dictionary of keywords"""
    keymap = {}
    for k in string_dict:
        keymap[k] = lp.keyword(k, str(string_dict[k]))
    return keymap


def all_types_to_keymap(input_config):
    """Convert all types of configs to keymap"""
    keymap = {}
    for k in input_config:
        if type(input_config[k]) == lp.keyword:
            keymap[k] = input_config[k]
        else:
            keymap[k] = lp.keyword(k, str(input_config[k]))
    return keymap



import numpy as np
from matplotlib import pylab as plt

from ._lephare import HIGH_CHI2, PDF
from ._utils import continueClass

__all__ = [
    "PDF",
]


@continueClass
class PDF:  # noqa
    def setYvals(self, yvals, is_chi2=False):  # noqa: N802
        if is_chi2:
            yvals[yvals >= HIGH_CHI2] = HIGH_CHI2
            self.chi2 = yvals
            yvals2 = np.exp(-0.5 * yvals)
            yvals2[yvals >= HIGH_CHI2] = 0.0
            self.vPDF = yvals2
        else:
            if np.any(np.array(yvals) < 0):
                raise ValueError("input array has negative values")
            self.vPDF = yvals
            yvals2 = np.zeros_like(yvals)
            yvals2[yvals == 0.0] = HIGH_CHI2
            yvals2[yvals != 0.0] = -2 * np.log(yvals[yvals != 0])
            self.chi2 = yvals2

    def plot(self, chi2=False, kwargs={}):  # noqa: B006
        if chi2:
            plt.plot(self.xaxis, self.chi2, **kwargs)
        else:
            plt.plot(self.xaxis, self.vPDF, **kwargs)



import numpy as np
from lephare._lephare import blackbody, check_first_char, indexes_in_vec


def test_globals_first_char():
    assert check_first_char("test")
    assert check_first_char("   test")
    assert not check_first_char("")
    assert not check_first_char(" ")
    assert not check_first_char("!test")
    assert not check_first_char("#test")
    assert not check_first_char("\t#")


def test_globals_blackbody():
    assert np.isclose(blackbody(10000, 500), 1.018807e-26)


def test_indexes_in_vec():
    v = [0.5, 0.9, 2.9, 3.5]
    val = 0.8
    assert indexes_in_vec(val, v, 0.1) == [1]
    assert indexes_in_vec(val, v, 0.01) == []
    assert indexes_in_vec(val, v, 0.3) == [0, 1]



import lephare as lp
import matplotlib
import numpy as np
import pytest

matplotlib.use("Agg")


def test_quadratic_extremum():
    # check alignment assertion
    def line(x):
        return 2 * x + 1

    x = [0, 1, 2]
    y = [line(t) for t in x]
    with pytest.raises(ValueError):
        xm, ym = lp.quadratic_extremum(*x, *y)

    for a in [+1, -1]:

        def parab(x, a):
            return a * x * x

        x = [-1, 0.5, 1]
        y = [parab(t, a) for t in x]
        xm, ym = lp.quadratic_extremum(*x, *y)
        assert xm == pytest.approx(0)
        assert ym == pytest.approx(0)

    # check for both concave and convex (min and max search)
    for a in [-2, 2]:
        b = 1
        c = 2
        x1 = 0.5

        def parab(x, a, b, c):
            return a * x**2 + b * x + c

        x = [x1 - 0.1, x1, x1 + 0.3]
        y = [parab(t, a, b, c) for t in x]
        xm, ym = lp.quadratic_extremum(*x, *y)
        assert xm == pytest.approx(-b / 2.0 / a)
        assert ym == pytest.approx(c - b**2 / 4.0 / a)

    # random parabola and random triplet
    p = -5 + 10 * np.random.random(3)  # a,b,c between -5 and 5

    def parab(x):
        return p[0] * x**2 + p[1] * b * x + p[2] * c

    xt = -p[1] / 2 / p[0]
    yt = parab(xt)
    x = xt - 0.1 + 0.2 * np.random.random(3)  # 3 pts at -0.1 +0.1 from extremum
    y = parab(x)
    xm, ym = lp.quadratic_extremum(*x, *y)
    assert xm == pytest.approx(xt)
    assert ym == pytest.approx(yt)


def test_pdf():
    """Basic test to ensure we can instantiate a PDF object."""
    test_pdf = lp.PDF(0, 3, 10)
    assert test_pdf.size() == 10
    assert len(test_pdf.chi2) == 10
    # testing bad input
    with pytest.raises(ValueError):
        test_pdf = lp.PDF(0, 3, 0)
    with pytest.raises(ValueError):
        test_pdf = lp.PDF(3, 0, 10)


def test_normalization():
    test_pdf = lp.PDF(0, 3.0, 101)
    yvals = np.linspace(0, 3.0, 101)
    test_pdf.setYvals(yvals, is_chi2=False)
    integral = test_pdf.normalization()
    assert integral == pytest.approx(4.5)
    integral2 = test_pdf.normalization()
    assert integral2 == pytest.approx(1)


def test_cumulant():
    pdf = lp.PDF(0.0, 1.0, 10)
    yvals = np.ones_like(pdf.xaxis)
    pdf.setYvals(yvals, is_chi2=False)
    cumulant = pdf.cumulant()
    np.testing.assert_almost_equal(cumulant, np.linspace(0, 1, 10))


def test_index():
    # 0, 0.33, 0.66,..., 2.66, 3 (10 values)
    pdf = lp.PDF(0, 3, 10)
    assert pdf.index(-1) == 0
    assert pdf.index(20) == 9
    assert pdf.index(2) == 6
    assert pdf.index(2.1) == 6
    assert pdf.index(2.2) == 7
    assert pdf.index(0) == 0
    assert pdf.index(3) == 9


def test_set_yvals():
    """Test to make sure that chi2 and vPDF are set correctly."""
    test_pdf = lp.PDF(0, 3, 10)
    yvals = np.linspace(0.001, 3, 10)
    test_pdf.setYvals(yvals, is_chi2=True)
    np.testing.assert_array_equal(test_pdf.chi2, yvals)
    np.testing.assert_array_almost_equal(test_pdf.vPDF, np.exp(-0.5 * yvals))

    test_pdf.setYvals(yvals, is_chi2=False)
    np.testing.assert_array_equal(test_pdf.vPDF, yvals)
    np.testing.assert_array_almost_equal(test_pdf.chi2, -2 * np.log(yvals))

    # checking that negative value in vPDF raises an exception
    with pytest.raises(ValueError):
        yvals[5] = -1
        test_pdf.setYvals(yvals, is_chi2=False)


def test_plot():
    """Simple test to exercise the plot method."""
    test_pdf = lp.PDF(0, 3, 10)
    yvals = np.linspace(0.001, 3, 10)
    test_pdf.setYvals(yvals, is_chi2=True)
    test_pdf.plot(chi2=True)
    test_pdf.plot(chi2=False)

    test_pdf.setYvals(yvals, is_chi2=False)
    test_pdf.plot(chi2=True)
    test_pdf.plot(chi2=False)


def test_credible_interval():
    test_pdf = lp.PDF(0, 1, 100)
    test_pdf.setYvals(np.ones_like(test_pdf.xaxis), is_chi2=False)
    # check output if argument val is outside of the xgrid range
    a, b = test_pdf.credible_interval(0.2, -1)
    assert a == -1
    assert b == -1
    a, b = test_pdf.credible_interval(0.2, 2)
    assert a == 2
    assert b == 2
    a, b = test_pdf.credible_interval(0.0, 2)
    assert a == 2
    assert b == 2
    # case where, even as a percentage, level is too high
    a, b = test_pdf.credible_interval(250, 0.5)
    assert a == pytest.approx(0.0)
    assert b == pytest.approx(1)
    # standard cases in the case of a trivial pdf
    a, b = test_pdf.credible_interval(0.2, 0.5)
    assert a == pytest.approx(0.4)
    assert b == pytest.approx(0.6)
    a, b = test_pdf.credible_interval(0.2, 0.9)
    assert a == pytest.approx(0.8)
    assert b == pytest.approx(1.0)
    a, b = test_pdf.credible_interval(0.2, 0.1)
    assert a == pytest.approx(0)
    assert b == pytest.approx(0.2)
    a, b = test_pdf.credible_interval(0.2, 0.0)
    assert a == pytest.approx(0)
    assert b == pytest.approx(0.2)
    # more reasonable pdf
    test_pdf = lp.PDF(0, 10, 1000)

    def gaus(x):
        return np.exp(-((x - 5.0) ** 2) / 2.0)

    test_pdf.setYvals(gaus(np.array(test_pdf.xaxis)), is_chi2=False)
    a, b = test_pdf.credible_interval(68.26, 5)
    assert a == pytest.approx(4, 0.01)
    assert b == pytest.approx(6, 0.01)
    a, b = test_pdf.credible_interval(95.46, 5)
    assert a == pytest.approx(3, 0.01)
    assert b == pytest.approx(7, 0.01)

    # analytical pdf
    x = np.linspace(-1, 1, 10001)
    y = np.zeros_like(x)
    y[x >= 0] = -x[x >= 0] + 1
    y[x < 0] = x[x < 0] + 1
    # triangular symmetric, area=1
    pdf = lp.PDF(-1, 1, 10001)
    pdf.setYvals(y, is_chi2=False)
    # central value
    val = 0.0
    for level in np.linspace(0.0, 1.0, 100):
        a, b = pdf.credible_interval(level, val)
        assert a == pytest.approx(-b)
        assert b == pytest.approx(1 - np.sqrt(1 - level))
    for val in [-0.5, -0.3, -0.1, 0.1, 0.3, 0.5]:
        for level in [0.1, 0.3, 0.6, 0.9]:
            a, b = pdf.credible_interval(level, val)
            cumul_left = 0.5 - val**2 / 2.0 + val if val > 0 else 0.5 + val**2 / 2.0 + val
            if cumul_left < level / 2.0:
                assert a == -1.0
                if level < 0.5:
                    assert b == pytest.approx(-1 + np.sqrt(2 * level))
                else:
                    assert b == pytest.approx(1 - np.sqrt(2 - 2 * level))
            if cumul_left > 1.0 - level / 2.0:
                assert b == 1.0
                if level < 0.5:
                    assert a == pytest.approx(1 - np.sqrt(2 * level))
                else:
                    assert a == pytest.approx(-1 + np.sqrt(2 - 2 * level))


def test_credible_interval2():
    pdf = lp.PDF(0, 1, 1000)
    pdf.setYvals(np.linspace(2, 1, len(pdf.xaxis)), is_chi2=False)

    # symetric case
    v = 0.5
    for level in [0.01, 0.1, 0.2, 0.34, 0.5]:
        lo, up = pdf.credible_interval(level, v)
        assert lo == pytest.approx(2 - np.sqrt((2 - v) ** 2 + 1.5 * level), 1.0e-4)
        assert up == pytest.approx(2 - np.sqrt((2 - v) ** 2 - 1.5 * level), 1.0e-4)

    # asymetric right case
    v = 0.1
    for level in [0.01, 0.1, 0.2, 0.34, 0.5]:
        lo, up = pdf.credible_interval(level, v)
        if 2 - np.sqrt((2 - v) ** 2 + 1.5 * level) < 0:
            assert lo == 0.0
            assert up == pytest.approx(2 - np.sqrt(4 - 3 * level), 1.0e-4)

    v = 0.9
    for level in [0.01, 0.1, 0.2, 0.34, 0.5]:
        lo, up = pdf.credible_interval(level, v)
        if 0.5 * (2 - v) ** 2 - 0.5 > 1:
            assert up == 0.0
            assert lo == pytest.approx(2 - np.sqrt(3 * level + 1))


def test_credible_interval3():
    # this is a case that caused problems in the past
    pdf = lp.PDF(3, 12.95, 200)
    yvals = np.zeros_like(pdf.xaxis)
    yvals[[119, 122, 123, 124, 126, 127]] = [
        1.90009e01,
        5.56300e-01,
        3.54100e-01,
        8.18000e-02,
        6.70000e-03,
        3.00000e-04,
    ]

    pdf.setYvals(yvals, is_chi2=False)
    a, b = pdf.credible_interval(0.68, 8.9526)
    assert a == pytest.approx(8.9168, 1.0e-4)
    assert b == pytest.approx(8.9884, 1.0e-4)


def test_levelcumu2x():
    pdf = lp.PDF(0, 1, 1000)
    pdf.setYvals(np.linspace(2, 1, len(pdf.xaxis)), is_chi2=False)
    assert pdf.levelCumu2x(0.5) == pytest.approx(2 - np.sqrt(5 / 2))


def test_improve_extremum():
    pdf = lp.PDF(0, 1, 101)
    xaxis = np.array(pdf.xaxis)
    for arg in [True, False]:
        a, b = pdf.improve_extremum(arg)
        assert a == pdf.xaxis[0]
        if arg:
            assert b == lp.HIGH_CHI2
        else:
            assert b == 0.0

    # tweak chi2 to check that parabolic improvement is
    # not done if one of the points is at HIGH_CHI2
    yval = np.array(pdf.chi2)
    yval[50] = 1
    yval[51] = 2
    pdf.setYvals(yval, is_chi2=True)
    a, b = pdf.improve_extremum(True)
    assert a == pdf.xaxis[50]
    assert b == 1
    # tweak vPDF to the same effect, with one of the points at 0
    yval = np.zeros_like(pdf.xaxis)
    yval[50] = 1
    yval[51] = 2
    pdf.setYvals(yval, is_chi2=False)
    a, b = pdf.improve_extremum(False)
    assert a == pdf.xaxis[51]
    assert b == 2

    # 2 normal situations
    pdf.setYvals((xaxis - 0.45) ** 2 + 0.2, True)
    a, b = pdf.improve_extremum(True)
    assert a == pytest.approx(0.45)
    assert b == pytest.approx(0.2)

    y = -((xaxis - 0.45) ** 2) + 0.2
    y[y < 0] = 0.0
    pdf.setYvals(y, False)
    a, b = pdf.improve_extremum(False)
    assert a == pytest.approx(0.45)
    assert b == pytest.approx(0.2)

    # Problematic situations
    pdf = lp.PDF(0, 1, 101)
    xaxis = np.array(pdf.xaxis)
    # extrememum at upper boundary
    pdf.setYvals((xaxis - 1.1) ** 2 + 0.2, True)
    a, b = pdf.improve_extremum(True)
    assert a == 1.0
    assert b == pytest.approx((a - 1.1) ** 2 + 0.2)
    pdf.setYvals(-((xaxis - 1.1) ** 2) + 2, False)
    a, b = pdf.improve_extremum(False)
    assert a == 1.0
    assert b == pytest.approx(-((a - 1.1) ** 2) + 2)
    # extrememum at lower boundary
    pdf = lp.PDF(0, 1, 101)
    xaxis = np.array(pdf.xaxis)
    # extrememum at upper boundary
    pdf.setYvals((xaxis) ** 2 + 0.2, True)
    a, b = pdf.improve_extremum(True)
    assert a == 0.0
    assert b == pytest.approx((a) ** 2 + 0.2)
    pdf.setYvals(-((xaxis) ** 2) + 2, False)
    a, b = pdf.improve_extremum(False)
    assert a == 0.0
    assert b == pytest.approx(-((a) ** 2) + 2)
    # HIGH_CHI2 issue
    pdf = lp.PDF(0, 1, 101)
    xaxis = np.array(pdf.xaxis)
    for i in range(3):
        # chi2 and a val at HIGH_CHI2
        yvals = (xaxis - 0.5) ** 2 + 0.2
        yvals[50 - 1 + i] = lp.HIGH_CHI2
        pdf.setYvals(yvals, True)
        a, b = pdf.improve_extremum(True)
        expected_min = 0.49 if i == 1 else 0.5
        assert a == pytest.approx(expected_min)
        assert b == pytest.approx((a - 0.5) ** 2 + 0.2)
        # vPDF and a val at 0
        yvals = -((xaxis - 0.5) ** 2) + 0.3
        yvals[50 - 1 + i] = 0
        pdf.setYvals(yvals, False)
        a, b = pdf.improve_extremum(False)
        expected_min = 0.49 if i == 1 else 0.5
        assert a == pytest.approx(expected_min)
        assert b == pytest.approx(-((a - 0.5) ** 2) + 0.3)
    # check the delta_x<0.5 condition
    pdf = lp.PDF(0, 1, 3)
    pdf.setYvals([0, 1, 0], False)
    assert a == 0.5
    assert b == 0.3  # normalized


def test_confidence_interval():
    pdf = lp.PDF(-5, 5, 1001)
    pdf.setYvals(np.array(pdf.xaxis) ** 2, is_chi2=True)
    a, b = pdf.confidence_interval(1)
    assert a == pytest.approx(-1)
    assert b == pytest.approx(1)
    a, b = pdf.confidence_interval(2.71)
    assert a == pytest.approx(-np.sqrt(2.71), 1.0e-5)
    assert b == pytest.approx(np.sqrt(2.71), 1.0e-5)



import os
import tempfile

import lephare as lp
import numpy as np
import pytest
from lephare import QSOSED, SED, GalSED, StarSED, flt

TESTDIR = os.path.abspath(os.path.dirname(__file__))
TESTDATADIR = os.path.join(TESTDIR, "../data")


def test_string_to_object():
    for t in ["s", "S", "sOap", "STAR"]:
        a = lp.SED.string_to_object(t)
        assert a == lp.object_type.STAR
    for t in ["g", "G", "GAGA", "GAL"]:
        a = lp.SED.string_to_object(t)
        assert a == lp.object_type.GAL
    for t in ["q", "Q", "QUASI", "QSO"]:
        a = lp.SED.string_to_object(t)
        assert a == lp.object_type.QSO
    with pytest.raises(ValueError):
        _ = lp.SED.string_to_object("wrong")


def sed_get_object_type():
    sed = SED("dummy", 0, "GAL")
    assert sed.get_object_type == lp.object_type.GAL
    sed = SED("dummy", 0, "QSO")
    assert sed.get_object_type == lp.object_type.QSO
    sed = SED("dummy", 0, "STAR")
    assert sed.get_object_type == lp.object_type.STAR


def test_sed_constructors():
    sed = SED("dummy", 10, "GAL")
    assert sed.name == "dummy"
    assert sed.nummod == 10
    assert sed.is_gal()
    sed2 = StarSED(sed)
    assert sed2.is_star()
    assert sed2.name == "dummy"
    assert sed2.nummod == 10
    sed2 = QSOSED(sed)
    assert sed2.is_qso()
    assert sed2.name == "dummy"
    assert sed2.nummod == 10
    sed2 = GalSED(sed)
    assert sed2.is_gal()
    assert sed2.name == "dummy"
    assert sed2.nummod == 10


def test_sed_fromfile():
    sed = SED("toto", 10, "GAL")
    sed.read(os.path.join(TESTDATADIR, "sed/o5v.sed.ext"))
    assert sed.size() == 4860
    d = sed.data()
    y = d[1]
    sed.rescale(2.0)
    y2 = sed.data()[1]
    assert np.allclose(y2, 2 * y)


def test_sed_readwrite():
    star = StarSED("star_test")
    star.read(os.path.join(TESTDATADIR, "sed/WDgd71.sed.ext"))

    with tempfile.TemporaryDirectory() as dir_name:
        file_path = os.path.join(dir_name, "test_star.fits")
        star.writeSED(file_path + ".bin", file_path + ".phys", file_path + ".doc")

        newsed = StarSED("newsed")
        newsed.readSEDBin(file_path + ".bin")
        np.testing.assert_array_equal(newsed.data()[0], star.data()[0])
        np.testing.assert_array_equal(newsed.data()[1], star.data()[1])


def test_emplace_back():
    sed = SED("toto", 10, "GAL")
    sed.emplace_back(100, 1)
    assert sed.size() == 1
    assert sed.lamb_flux[0].lamb == 100
    assert sed.lamb_flux[0].val == 1
    assert sed.lamb_flux[0].ori == 1


def test_set_vector():
    sed = SED("toto", 10, "GAL")
    x = np.linspace(100, 500, 1000)
    with pytest.raises(RuntimeError):
        # not the same size
        sed.set_vector(x, np.linspace(0, 1, 500))
    sed.set_vector(x, np.ones_like(x))
    assert len(x) == sed.size()
    for el in sed.lamb_flux:
        assert el.val == 1
        assert el.ori == 1
    # test clearing
    sed.set_vector(x, x)
    assert len(x) == sed.size()


def test_integration():
    sed = SED("toto", 10, "GAL")
    # Tophat filter
    lmin = 200
    lmax = 300
    hat = flt(lmin, lmax, 100)
    # flat SED ov val=1
    x = np.linspace(100, 500, 1000)
    sed.set_vector(x, np.ones_like(x))
    result = sed.integrateSED(hat)
    c = 2.99792458e18
    true_res = [
        lmax - lmin,
        c * (1 / lmin - 1 / lmax),
        0.5 * (lmax**2 - lmin**2),
        lmax - lmin,
        0.5 * (lmax**2 - lmin**2),
        1 / lmin - 1 / lmax,
    ]
    print(result)
    print(true_res)
    assert np.allclose(np.array(result), np.array(true_res), 1.1e-2)



import os

from lephare import GalMag

TESTDIR = os.path.abspath(os.path.dirname(__file__))
TESTDATADIR = os.path.join(TESTDIR, "../data")


def test_readb12():
    mag = GalMag()
    mag.set_zgrid(0.5, 0, 1)
    mag.read_B12()



import os

import lephare as lp
import numpy as np
import pytest
import scipy.integrate as sciint


def test_filter_extinc():
    """Test the filter_extinc runner makes a file and has good values"""
    test_dir = os.path.abspath(os.path.dirname(__file__))
    os.environ["LEPHAREDIR"] = os.path.join(test_dir, "../data")
    os.environ["LEPHAREWORK"] = os.path.join(test_dir, "../tmp")
    out_file = os.path.join(test_dir, "../tmp", "filter_extinc.dat")
    options = {
        "FILTER_FILE": os.path.join(test_dir, "../data", "filt", "LSST_FILTERS.dat"),
        "EXT_CURVE": "SB_calzetti.dat",
        "GAL_CURVE": "CARDELLI",
        "OUTPUT": out_file,
    }
    runner = lp.FiltExt(config_keymap=lp.all_types_to_keymap(lp.default_cosmos_config), **options)
    all_filters, aint, albdav, albd = runner.run()

    # Check it made the file
    assert os.path.exists(out_file)
    with open(out_file, "r") as f:
        contents = f.read()
    assert float(contents.split()[-1]) == pytest.approx(1.3039147701114893)

    # check computation
    atmoext = lp.ext("atmo", 0)
    atmoext.read(os.path.join(os.environ["LEPHAREDIR"], "ext", "SB_calzetti.dat"))
    x1 = [o.lamb for o in atmoext.lamb_ext]
    y1 = [o.val for o in atmoext.lamb_ext]

    def ext(x):
        return np.interp(x, x1, y1, 0, 0)  # 0, 0 are the extrapolated default values

    aint2 = []
    # errs = []
    for f in all_filters:
        x2 = [o.lamb for o in f.lamb_trans]
        y2 = [o.val for o in f.lamb_trans]

        def filt(x):
            # 0, 0 are the extrapolated default values
            return np.interp(x, x2, y2, 0, 0)  # noqa: B023

        def fe(x):
            return ext(x) * filt(x)

        n = sciint.quad(fe, f.lmin(), f.lmax(), limit=200, epsabs=1.0e-3, epsrel=1.0e-4)
        d = sciint.quad(filt, f.lmin(), f.lmax(), limit=200, epsabs=1.0e-3, epsrel=1.0e-4)
        res = n[0] / d[0]
        aint2.append(res)
        # errs.append(sqrt(res**2 *(n[1]**2/n[0]**2 + d[1]**2/d[0]**2)) )

    np.testing.assert_array_almost_equal(aint, aint2, 5.0e-4)



# ruff: noqa: N999

from lephare import GalMag, QSOMag, StarMag, keyword, read_config

__all__ = [
    "MagSvc",
]


class MagSvc:
    """Magnitude service to return magnitudes of a single SED."""

    @classmethod
    def from_config(cls, objtype, config_file):
        """Load config and return magnitudes

        Parameters
        ----------
        objtype : `str`
            The type of object galaxy, star, or qso based on case insensitive first
            letter of string.
        config_file : `str`
            Path to config file.
        """
        keywords = [
            "COSMOLOGY",
            "FILTER_FILE",
            "MAGTYPE",
            "EXTINC_LAW",
            "EB_V",
            "MOD_EXTINC",
            "ZGRID_TYPE",
            "Z_STEP",
            "LIB_ASCII",
            "VERBOSE",
            "ADD_DUSTEM",
        ]
        if objtype[0].upper() == "G":
            keywords += ["GAL_LIB_IN", "GAL_LIB_OUT", "EM_LINES", "EM_DISPERSION", "ADD_DUSTEM"]
            instance = GalMag
        elif objtype[0].upper() == "Q":
            keywords += ["QSO_LIB_IN", "QSO_LIB_OUT"]
            instance = QSOMag
        elif objtype[0].upper() == "S":
            keywords += ["STAR_LIB_IN", "STAR_LIB_OUT"]
            instance = StarMag
        else:
            raise ValueError(f"Unknown objtype: {objtype}")

        keymap = read_config(config_file)
        keymap["t"] = keyword("t", objtype)
        keymap["c"] = keyword("c", config_file)
        return instance(keymap)



import time
from contextlib import suppress

from ._lephare import (
    PhotoZ,
    keyword,
)
from .runner import Runner

__all__ = [
    "Zphota",
]

config_keys = {
    "verbose": "increase onscreen verbosity",
    "CAT_IN": "input catalog",
    "INP_TYPE": "F if Fluxes, M if magnitudes",
    "CAT_MAG": "AB or VEGA magnitude system of the input catalog (default AB)",
    "CAT_FMT": "MEME or MMEE, to precise the order of value and error columns",
    "CAT_LINES": "min,max range to subselect entries in the input catalog",
    "ZPHOTLIB": "comma-separated list of template magnitude libraries (GAL, STAR, or QSO), \
    without the suffix (to be found in $LEPHAREWORK/lib_mag)",
    "ADD_EMLINES": "Range of galaxy models for which considering emission lines contribution.",
    "EBV_RANGE": "E(B-V) min and max allowed in the library. Applied to all attenuation laws",
    "Z_RANGE": "Z min and max allowed in the GALAXY library",
    "PARA_OUT": "file listing the output keywords to be written on file (one keyword per line)",
    "CAT_OUT": "output file name",
    "CAT_TYPE": "LONG or SHORT, depending on whether zspec, context, and other columns are present \
    after the magnitudes/fluxes and error columns",
    "ERR_SCALE": "values to be added in quadrature to the errors in magnitude in each band\
    separated by a comma",
    "ERR_FACTOR": "values to scale the errors in flux (a single value for all bands)",
    "GLB_CONTEXT": "global context to define the bands to be used in the fit",
    "FORB_CONTEXT": "forbidden context to define the bands to be discarded from the fit",
    "MAG_ABS": "min,max absolute magnitudes allowed for GAL type (prior)",
    "MAG_ABS_QSO": "min,max absolute magnitudes allowed for QSO/AGN type (prior)",
    "MAG_REF": "reference band to compute absolute magnitudes when using the prior",
    "NZ_PRIOR": "band to be used in the n(z) prior; a second band can be given\
    in case the first one is absent in some objects",
    "ZFIX": "fix z to a known redshift, in case one is interested in inferring the best template",
    "Z_INTERP": "perform quadratic interpolation to refine best redshift solution",
    "Z_METHOD": "which redshift solution, between best and pdf median,\
    to use for physical parameter computation",
    "DZ_WIN": "Window search for second peak",
    "MIN_THRES": "lower threshold for second peak",
    "SPEC_OUT": "output the best spectrum of each object (YES, or the name of the directory\
    to be used",
    "FIR_LIB": "dedicated far infrared synthetic magnitudes files",
    "FIR_LMIN": "minimum value for lambda_mean/(1+z) (in microns) in FIR analysis",
    "FIR_CONT": "context for the FIR bands to be used in the FIR fit",
    "FIR_SCALE": "context for the FIR scale determination in the FIR fit",
    "FIR_FREESCALE": "consider FIR scaling free",
    "FIR_SUBSTELLAR": "substract the stellar component to the observed IR fluxes",
    "MABS_METHOD": "Method to compute absolute magnitudes\
    (if 1 use the most appropriated filter for observed flux, 3 the model)",
    "MABS_CONTEXT": "Context to decide which band can be used to derive the\
    absolute magnitude in method 1",
    "MABS_REF": "Fixed filter used to derived all abs mag (if MABS_METHOD=2)",
    "MABS_FILT": "List of fixed filters chosen to derive abs mag in all bands depending on\
    the redshift bins defined in MABS_ZBIN (if MABS_METHOD=4)",
    "MABS_ZBIN": "List of redshift bins associated with MABS_FILT",
    "RF_COLORS": "list of 4 band indexes out of which to compute the probability\
    distribution of the 2 corresponding rest frame colors (default=-1 which means\
    to not compute these color distributions)",
    "M_REF": "band index for which to compute the rest frame absolute magnitude\
    probability distribution",
    "APPLY_SYSSHIFT": "list of values (equal to the number of bands) to add to the predicted\
    magnitudes as zero-points  (substracted to the observed magnitudes).\
    These are also the output of the auto adaptation stage",
    "AUTO_ADAPT": "perform zero point auto adaptation based on the provided true redshifts",
    "ADAPT_BAND": "band number for which to check magnitude range for auto adaptation",
    "ADAPT_LIM": "min,max magnitude range for auto adaptation",
    "ADAPT_ZBIN": "min,max value of the true redshifts used for auto adaptation of the zero points",
    "PDZ_OUT": "output the PDF of each object",
    "PDZ_TYPE": "list of the PDFs to output if PDZ_OUT is set",
    "ADDITIONAL_MAG": "file basename (in $LEPHAREWORK/filt/) for the set of additional filters\
    for which to compute magnitudes. Thus a pass to the LePHARE `filter` stage needs to have\
    occurred on these filters.",
    "LIMITS_ZBIN": "Used to compute z_max. Redshifts used to split in N bins, separated by a coma.\
    Need N+1 values (start with the minimum redshift).",
    "LIMITS_MAPP_REF": "Used to compute z_max. Band in which the absolute magnitude is computed",
    "LIMITS_MAPP_SEL": "Used to compute z_max. Give the selection band in each redshift bin.\
    Need 1 or N values.",
    "LIMITS_MAPP_CUT": "Used to compute z_max. Apparent mag selection used in each redshift bin.\
    Need 1 or N values.",
    "CHI2_OUT": "Flag to output the chi2 value of each template (one file per source)",
}

nonestring = "NONE"


class Zphota(Runner):
    """The specific arguments to the Zphota class are
    verbose:
           add verbosity
    CAT_IN:
           input catalog
    INP_TYPE:
           F if Fluxes, M if magnitudes
    CAT_MAG:
           AB or VEGA magnitude system of the input catalog (default AB)
    CAT_FMT:
           MEME or MMEE, to precise the order of value and error columns
    CAT_LINES:
           min,max range to subselect entries in the input catalog
    ZPHOTLIB:
           comma-separated list of template magnitude libraries \
without the suffix (to be found in $LEPHAREWORK/lib_mag)
    ADD_EMLINES:
           range of galaxy models for which considering emission lines contribution
    EBV_RANGE:
           E(B-V) min and max allowed in the library. Applied to all attenuation laws
    Z_RANGE:
           Z min and max allowed in the GALAXY library
    PARA_OUT:
           file listing the output keywords to be written on file (one keyword per line)
    CAT_OUT:
           output file name
    CAT_TYPE:
           LONG or SHORT, depending on whether zspec, context, and other columns are present \
after the magnitudes/fluxes and error columns
    ERR_SCALE:
           values to be added in quadrature to the errors in magnitude in each band, \
values separated by a comma
    ERR_FACTOR:
           values to scale the errors in flux (a single value for all bands)
    GLB_CONTEXT:
           global context to define the bands to be used in the fit
    FORB_CONTEXT:
           forbidden context to define the bands to be discarded from the fit
    MAG_ABS:
           min,max absolute magnitudes allowed for GAL type
    MAG_ABS_QSO:
           min,max absolute magnitudes allowed for QSO/AGN type
    MAG_REF:
           reference band to compute absolute magnitudes
    NZ_PRIOR:
           band to be used in the n(z) prior; a second band can be given \
in case the first one is absent in some objects
    ZFIX:
           fix z to a known redshift, in case one is interested in inferring the best template
    Z_INTERP:
           perform quadratic interpolation to refine best redshift solution
    Z_METHOD:
           which redshift solution, between best and pdf median, \
to use for physical parameter computation
    DZ_WIN:
           Window search for second peak
    MIN_THRES:
           lower threshold for second peak
    SPEC_OUT:
           output the best spectrum of each object
    FIR_LIB:
           dedicated far infrared synthetic magnitudes files
    FIR_LMIN:
           minimum value for lambda_mean/(1+z) (in microns) in FIR analysis
    FIR_CONT:
           context for the FIR bands to be used in the FIR fit
    FIR_SCALE:
           context for the FIR scale determination in the FIR fit
    FIR_FREESCALE:
           consider FIR scaling free
    FIR_SUBSTELLAR:
           substract the stellar component to the observed IR fluxes
    MABS_METHOD:
           method to compute absolute magnitudes \
(e.g.  1 use the most appropriated filter for observed flux, 3 the model)
    MABS_CONTEXT:
           context to decide which band can be used to derive the \
absolute magnitude in method 1
    MABS_REF:
           fixed filter used to derived all abs mag (if MABS_METHOD=2)
    MABS_FILT:
           list of fixed filters chosen to derive abs mag in all bands depending on \
the redshift bins defined in MABS_ZBIN (if MABS_METHOD=4)
    MABS_ZBIN:
           list of Redshift bins associated with MABS_FILT
    RF_COLORS:
           list of 4 band indexes out of which to compute the probability \
distribution of the 2 corresponding rest frame colors (default=-1 which means \
to not compute these color distributions)
    M_REF:
           band index for which to compute the rest frame absolute magnitude \
probability distribution
    APPLY_SYSSHIFT:
           list of values (equal to the number of bands) to add to the predicted \
magnitudes as zero-points  (substracted to the observed magnitudes)
    AUTO_ADAPT:
           perform zero point auto adaptation based on the provided true redshifts
    ADAPT_BAND:
           band number for which to check magnitude range for auto adaptation
    ADAPT_LIM:
           min,max magnitude range for auto adaptation
    ADAPT_ZBIN:
           min,max value of the true redshifts used for auto adaptation of the zero points
    PDZ_OUT:
           output the PDF of each object
    PDZ_TYPE:
           list of the PDFs to output if PDZ_OUT is set
    ADDITIONAL_MAG:
           file basename (in $LEPHAREWORK/filt/) for the set of additional filters \
for which to compute magnitudes. Thus a pass to the LePHARE `filter` stage needs to have \
occurred on these filters.
    LIMITS_ZBIN:
            used to compute z_max. Redshifts used to split in N bins, separated by a coma. \
Need N+1 values (start with the minimum redshift).
    LIMITS_MAPP_REF:
            used to compute z_max. Band in which the absolute magnitude is computed
    LIMITS_MAPP_SEL:
            used to compute z_max. Give the selection band in each redshift bin. \
Need 1 or N values.
    LIMITS_MAPP_CUT:
            used to compute z_max. Apparent mag selection used in each redshift bin. \
Need 1 or N values
    CHI2_OUT
            Flag to output the chi2 value of each template (one file per source)
    """

    def update_help(self):
        """Add the specific Zphota help"""
        doc = "Execute LePHARE photo-z estimation"
        with suppress(Exception):
            self.parser.usage = doc
        self.__doc__ = doc + "\n"  # + inspect.getdoc(Zphota)

    def __init__(self, config_file=None, config_keymap=None, **kwargs):
        super().__init__(config_keys, config_file, **kwargs)

    def run(self, **kwargs):
        super().run(**kwargs)

        self.keymap["c"] = keyword("c", self.config)

        photoz = PhotoZ(self.keymap)

        # Compute offsets depending on the AUTO_ADAPT and APPLY_SYSSHIFT options (0 if none)
        adapt_srcs = photoz.read_autoadapt_sources()
        a0 = photoz.compute_offsets(adapt_srcs)

        fit_srcs = photoz.read_photoz_sources()
        photoz.run_photoz(fit_srcs, a0)
        photoz.write_outputs(fit_srcs, int(time.time()))

        self.photoz = photoz
        return

    def end(self):
        super().end()


def main():  # pragma no cover
    runner = Zphota()
    runner.run()
    runner.end()


if __name__ == "__main__":  # pragma no cover
    main()



from matplotlib import pylab as plt

from ._lephare import flt
from ._utils import continueClass

__all__ = [
    "flt",
]


@continueClass
class flt:  # noqa
    def plot_filter_curve(self, normed=False):
        filter_name = self.name
        plt.title(filter_name)
        plt.xlabel("wavelength")
        plt.ylabel("value")
        x = self.data()[0]
        y = self.data()[1]
        if normed:
            y /= y.max()
        plt.plot(x, y, label="curve")
        plt.plot([self.lambdaMean(), self.lambdaMean()], [0.9 * y.min(), 1.1 * y.max()], label="lambda mean")
        plt.plot([self.lambdaEff(), self.lambdaEff()], [0.9 * y.min(), 1.1 * y.max()], label="lambda eff")
        plt.legend()



import os

from lephare import _lephare


def test_one_ei_lambda_constructor():
    el1 = _lephare.oneElLambda(10, 10, 0)
    # test that 1 + 1 = 2
    assert el1.lamb == 10
    assert el1.val == 10
    assert el1.ori == 0
    el2 = _lephare.oneElLambda(el1)
    assert el1.lamb == el2.lamb
    assert el1.val == el2.val
    assert el1.ori == el2.ori


def test_one_ei_lambda_interp():
    el1 = _lephare.oneElLambda(10, 10, 0)
    el2 = _lephare.oneElLambda(20, 20, 0)
    el3 = _lephare.oneElLambda(15, 0, 0)
    el3.interp(el1, el2)
    assert el3.val == 15


def test_one_ei_lambda_constructor2():
    ext = _lephare.ext(name="test", numext=10)
    assert ext.name == "test"
    assert ext.numext == 10
    assert len(ext.lamb_ext) == 0


def test_one_ei_lambda_read_calzetti(test_data_dir):
    filename = os.path.join(test_data_dir, "calzetti.dat")
    ext = _lephare.ext(name="test", numext=10)
    ext.read(filename)
    assert ext.lmin == 400.0
    assert ext.lmax == 40000.0


def test_one_ei_lambda_read_tau10(test_data_dir):
    filename = os.path.join(test_data_dir, "tau10.out")
    ext = _lephare.ext(name="test", numext=10)
    ext.read(filename)



import os
from contextlib import suppress

import numpy as np

import lephare as lp

from ._lephare import GalMag, compute_filter_extinction, ext
from .runner import Runner

__all__ = [
    "FiltExt",
    "calculate_extinction_values",
]

config_keys = {
    "verbose": "increase onscreen verbosity",
    "FILTER_FILE": "path to filter file on which to compute extinction,\
    output of the `filter` execution, to be found in $LEPHAREWORK/filt",
    "EXT_CURVE": "extinction law to use, to be searched in $LEPHAREDIR/ext if relative",
    "GAL_CURVE": "extinction curve in the galaxy, to be searched in $LEPHAREDIR/ext if relative",
    "OUTPUT": "output file name",
}


class FiltExt(Runner):
    """
    The specific arguments to the Filter class are

    verbose
        increase onscreen verbosity
    FILTER_FILE
        Path to filter file on which to compute extinction, output of the
        `filter` execution, to be found in $LEPHAREWORK/filt
    EXT_CURVE
        Extinction law to use, to be searched in $LEPHAREDIR/ext if relative
    GAL_CURVE
        Extinction curve in the galaxy, to be searched in $LEPHAREDIR/ext if relative
    OUTPUT
        Output file name
    """

    def update_help(self):
        """Add specific help information"""
        doc = "Compute atmospheric extinction for a given filter\n"
        with suppress(Exception):
            self.parser.usage = doc
        self.__doc__ = doc + "\n"

    def __init__(self, config_file=None, config_keymap=None, **kwargs):
        super().__init__(config_keys, config_file, config_keymap, **kwargs)

    def run(self, **kwargs):
        """Update keymap and verbosity based on call arguments.
        This is only when the code is called from python session.
        """
        super().run(**kwargs)
        keymap = self.keymap
        # Get the parameters
        filters = keymap["FILTER_FILE"].split_string("unknown", 1)[0]
        if not os.path.isabs(filters):
            filters = os.path.join(os.environ["LEPHAREWORK"], "filt", filters)
        atmec = keymap["EXT_CURVE"].split_string("NONE", 1)[0]
        output = keymap["OUTPUT"].split_string("filter_extinc.dat", 1)[0]
        galec = keymap["GAL_CURVE"].split_string("CARDELLI", 1)[0]

        all_filters, aint, albdav, albd = calculate_extinction_values(
            filters, atmec, galec, verbose=self.verbose
        )

        if self.verbose:
            print("#######################################")
            print("# Computing ATMOSPHERIC AND GALACTIC EXTINCTION ")
            print("# with the following options:")
            print(f"# Config file: {self.config}")
            print(f"# FILTER_FILE: {filters}")
            print(f"# EXT_CURVE (Atmospheric extinction curve): {atmec}")
            print(f"# GAL_CURVE (Galactic extinction curve: {galec}")
            print(f"# Output file: {output}")
            print("#######################################")
            print(" Filters Ext(mag/airmass) Albda/Av Albda/E(B-V) ")
            for k, f in enumerate(all_filters):
                name = os.path.basename(f.name)
                print(f"{name:20} {aint[k]:<20} {albdav[k]:<20} {albd[k]:<20}")

        with open(output, "w") as out:
            out.write("#######################################\n")
            out.write("# Computing ATMOSPHERIC AND GALACTIC EXTINCTION \n")
            out.write("# with the following options:\n")
            out.write(f"# Config file: {self.config}\n")
            out.write(f"# FILTER_FILE: {filters}\n")
            out.write(f"# EXT_CURVE (Atmospheric extinction curve): {atmec}\n")
            out.write(f"# GAL_CURVE (Galactic extinction curve: {galec}\n")
            out.write(f"# Output file: {output}\n")
            out.write("#######################################\n")
            out.write(" Filters Ext(mag/airmass) Albda/Av Albda/E(B-V) \n")
            for k, f in enumerate(all_filters):
                name = os.path.basename(f.name)
                out.write(f"{name:20} {aint[k]:<20} {albdav[k]:<20} {albd[k]:<20}\n")

        return all_filters, aint, albdav, albd


def calculate_extinction_values(filters, atmec, galec, verbose=False):
    """Calculate the extinction values for a set of filters

    Parameters
    ==========

    filters :  str
        The file containing the lephare compiled filter list
    atmec : str
        Extinction law to use, to be searched in $LEPHAREDIR/ext if relative
    galec : str
        Extinction curve in the galaxy, to be searched in $LEPHAREDIR/ext if relative
    verbose : bool
        Increase onscreen verbosity

    Returns
    =======

    all_filters : list of lephare.GalMag
        The list of filter objects
    aint : np.array
        Atmospheric extinction in each filter (mag/airmass)
    albdav : np.array
        Galactic extinction in each filter A(lbd)/Av
    albd : np.array
        Galactic extinction in each filter A(lbd)/E(B-V)
    """
    all_filters = GalMag.read_flt(filters)
    if atmec == "NONE":
        aint = np.full(len(all_filters), 99.0).tolist()
    else:
        if not os.path.isabs(atmec):
            atmec = os.path.join(os.environ["LEPHAREDIR"], "ext", atmec)
        atmospheric_ext = ext(atmec, 0)
        atmospheric_ext.read(atmec)
        aint = [compute_filter_extinction(f, atmospheric_ext) for f in all_filters]

    if galec == "CARDELLI":
        # If cardelli (hardcoded) with Rv=3.1 by  default
        # output A(lbd)/Av->A(lbd)/(Rv*E(B-V))->A(lbd)/E(B-V)=Rv*A(lbd)/Av
        albdav = np.array([lp.cardelli_ext(f) for f in all_filters])
        albd = albdav * 3.1
    else:
        if not os.path.isabs(galec):
            galec = os.path.join(os.environ["LEPHAREDIR"], "ext", galec)
        galactic_ext = ext(galec, 1)
        galactic_ext.read(galec)

        #  Rv=3.1 except for Calzetti law (4.05) and SMC Prevot (2.72)
        rv = 3.1
        if "SMC_prevot" in galec:
            rv = 2.72
            if "calzetti" in galec:
                rv = 4.05
        if verbose:
            print(f"assuming Rv={rv} for this Extinction law {galec}")
        # galactic curves given in k(lbda) (=A(lbda)/E(B-V))
        #  -> A(lbda)/Av = A(lbda)/E(B-V) / Rv)
        albd = np.array([compute_filter_extinction(f, galactic_ext) for f in all_filters])
        albdav = albd / rv
    return all_filters, aint, albdav, albd


def main():  # pragma no cover
    runner = FiltExt()
    runner.run()
    runner.end()


if __name__ == "__main__":  # pragma no cover
    main()



# ruff: noqa: N999

import os
import xml.dom.minidom

import requests
import yaml

from lephare import LEPHAREDIR

from ._lephare import check_first_char, flt

__all__ = [
    "FilterSvc",
]

SVO_URL = "http://svo2.cab.inta-csic.es/theory/fps"


class FilterSvc:
    """Filter service for retrieving filters from various sources.

    This class defines a number of methods for loading and manipulating filters.
    """

    @classmethod
    def from_yaml(cls, yaml_file):
        """Load filter from a yaml file

        Parameters
        ----------
        yaml_file : `str`
            Path to yaml file

        Returns
        -------
        flt_array :  list of `lephare.flt`
            Array of wavelengths in Angstrom and filter transmissivity.
        """
        config = yaml.load(open(yaml_file), Loader=yaml.BaseLoader)["filters"]  # noqa: SIM115
        if "calib" in config:
            default_calib = config["calib"]
        if "trans" in config:
            default_trans = config["trans"]
        flt_array = []
        counter = 0
        for entry in config["list"]:
            name = entry["name"]
            # filters are ordered starting from one (FORTRAN legacy)
            counter += 1  # noqa: SIM113
            calib = entry.get("calib", default_calib)
            trans = entry.get("trans", default_trans)
            if name[:4] == "svo:":
                flt_obj = cls.from_svo(counter, name[4:], "AB", calib)
            else:
                flt_obj = cls.from_file(name, counter, trans, calib)
            flt_array.append(flt_obj)
        return flt_array

    @classmethod
    def from_config(cls, config_file):
        """Load filter from a .para config file

        Parameters
        ----------
        config_file : `str`
            Path to config file

        Returns
        -------
        flt_array : list of `lephare.flt`
            List of filters.
        """
        keywords = ["FILTER_REP", "FILTER_LIST", "TRANS_TYPE", "FILTER_CALIB", "FILTER_FILE"]
        keymap = {}
        with open(config_file) as fstream:
            count = 0
            while count < len(keywords):
                line = fstream.readline()
                for key in keywords:
                    if key in line and check_first_char(line):
                        keymap[key] = line.split()[1]
                        count += 1
        keymap["FILTER_REP"] = keymap["FILTER_REP"].replace("$LEPHAREDIR", LEPHAREDIR)

        filter_list = keymap["FILTER_LIST"].split(",")
        filter_calib = keymap["FILTER_CALIB"].split(",")
        filter_trans = keymap["TRANS_TYPE"].split(",")
        if len(filter_trans) == 1:
            filter_trans = len(filter_list) * filter_trans
        elif len(filter_trans) != len(filter_list):
            raise RuntimeError("FILTER_LIST and FILTER_TRANS do not have the same size")
        if len(filter_calib) == 1:
            filter_calib = len(filter_list) * filter_calib
        elif len(filter_calib) != len(filter_list):
            raise RuntimeError("FILTER_LIST and FILTER_CALIB do not have the same size")
        flt_array = []
        for i in range(len(filter_list)):
            name = os.path.join(keymap["FILTER_REP"], filter_list[i])
            oneflt = flt(i, name, int(filter_trans[i]), int(filter_calib[i]))
            flt_array.append(oneflt)
        return flt_array

    @classmethod
    def from_svo(cls, counter, filter_id, system="AB", calib=0):
        """Return filter from SVO

        Parameters
        ----------
        counter : `int`
            Filter number
        filter_id : `str`
            Id of filter in SVO format.
        system : `str`, optional
            Photometric system
        calib : `float`, optional
            Calibration value. Not currently used.

        Returns
        -------
        res : `lephare.flt`
            Filter in native lephare format.
        """
        res = FilterSvc.svo_request(counter, filter_id, system)
        return res

    @classmethod
    def from_file(cls, filename, counter=-1, trans=0, calib=0):
        """Return filter from SVO

        Parameters
        ----------
        filename : `str`
            Path to filter file.
        counter : `int`
            Filter number
        trans : `int`, optional
            Photometric system.
        calib : `int`, optional
            Calibration value.

        Returns
        -------
        f : `lephare.flt`
            Filter in native lephare format.
        """
        name = filename.replace("$LEPHAREDIR", LEPHAREDIR)
        f = flt(counter, name, trans, calib)
        return f

    @classmethod
    def svo_request(cls, counter, filter_id, system):
        """Retrieve a filter from the SVO

        Parameters
        ----------
        counter : `int`
            Filter number
        filter_id : `str`
            Id of filter in SVO format.
        system : `str`, optional
            Photometric system

        Returns
        -------
        res : `lephare.flt`
            Filter in native lephare format.
        """
        try:
            query = f"{SVO_URL}/fps.php?PhotCalID={filter_id}/{system}"
            r = requests.get(query, timeout=5)
        except ConnectionRefusedError:  # pragma no cover
            print(f"request {query} failed due to failure to connect to the server.")
            return None
        except requests.ConnectTimeout:  # pragma no cover
            print("Timeout on SVO server")
            return None
        try:
            dd = xml.dom.minidom.parseString(r.content)
        except xml.parsers.expat.ExpatError:
            print("SVO server down")
            return None
        # assert query ok
        for info in dd.getElementsByTagName("INFO"):
            if info.getAttribute("name") == "QUERY_STATUS" and info.getAttribute("value") != "OK":
                raise AssertionError(
                    f"QUERY_STATUS did not return OK; check the filter_id input: {SVO_URL}. "
                    f"For a list of valid filters, see {filter_id}"
                )
        # filter params: not used (yet?)
        params = dd.getElementsByTagName("PARAM")
        param_dict = {"Date": r.headers["Date"]}
        for param in params:
            param_dict[param.getAttribute("name")] = param.getAttribute("value")

        trans_type = int(param_dict["DetectorType"])
        name = filter_id.split("/")[1]

        # filter curve
        ##check units
        f1, f2 = dd.getElementsByTagName("FIELD")
        # these assert are meant to secure against non
        # standard entries in the SVO DB
        assert f1.getAttribute("unit") == "Angstrom"
        assert f1.getAttribute("datatype") == "double"
        assert f2.getAttribute("datatype") == "double"
        table = dd.getElementsByTagName("TABLEDATA")[0]
        data = table.getElementsByTagName("TR")
        with open("./" + name, "w") as stream:
            for _, d in enumerate(data):
                dd = d.getElementsByTagName("TD")
                l_val = float(dd[0].childNodes[0].data)
                t_val = float(dd[1].childNodes[0].data)
                stream.write(f"{l_val} {t_val}\n")

        # not super nice, but the only quick solution found to
        # avoid exposing trans and clean methods.
        flt_obj = flt(counter, "./" + name, trans_type, 0)
        # flt_obj.read("dummy")
        os.remove(name)
        # save the SVO additional info in an instance property
        flt_obj.svo_params = param_dict
        return flt_obj

    @classmethod
    def from_keymap(cls, keymap):
        """Load filter from a config keymap.

        Parameters
        ----------
        keymap : dict of lephare.keyword
            The config keymap

        Returns
        -------
        flt_array : list of `lephare.flt`
            List of filters.
        """
        keymap["FILTER_REP"].value = keymap["FILTER_REP"].value.replace("$LEPHAREDIR", LEPHAREDIR)

        filter_list = keymap["FILTER_LIST"].value.split(",")
        filter_calib = keymap["FILTER_CALIB"].value.split(",")
        filter_trans = keymap["TRANS_TYPE"].value.split(",")
        if len(filter_trans) == 1:
            filter_trans = len(filter_list) * filter_trans
        elif len(filter_trans) != len(filter_list):
            raise RuntimeError("FILTER_LIST and FILTER_TRANS do not have the same size")
        if len(filter_calib) == 1:
            filter_calib = len(filter_list) * filter_calib
        elif len(filter_calib) != len(filter_list):
            raise RuntimeError("FILTER_LIST and FILTER_CALIB do not have the same size")
        flt_array = []
        for i in range(len(filter_list)):
            name = os.path.join(keymap["FILTER_REP"].value, filter_list[i])
            oneflt = flt(i, name, int(filter_trans[i]), int(filter_calib[i]))
            flt_array.append(oneflt)
        return flt_array



import os

import lephare as lp
import pytest
from lephare._lephare import keyword

TESTDIR = os.path.abspath(os.path.dirname(__file__))
TESTDATADIR = os.path.join(TESTDIR, "../data")


def test_runner_base():
    """This is the most rudimentary test of the Runner class. We have to provide
    both a list of config_keys and a path to a config file. We want to check only
    that the class can be instantiated and that the resulting keymap is not empty."""

    test_keys = {
        "CAT_IN": "help1",
    }
    config_file_path = os.path.join(TESTDATADIR, "examples/COSMOS.para")
    runner = lp.Runner(config_keys=test_keys, config_file=config_file_path)
    assert len(runner.keymap)
    runner = lp.Runner(config_keys=test_keys, config_keymap={"CAT_IN": keyword("CAT_IN", "dummy")})
    assert len(runner.keymap)
    runner = lp.Runner(config_keys=test_keys, CAT_IN="dummy")
    assert len(runner.keymap)
    runner = lp.Runner(config_keys=test_keys, cat_in="dummy")
    assert len(runner.keymap)


def test_runner_no_config_keys():
    """Expect that a RuntimeError is raised when no config_keys or
    config_keys=None is passed to Runner."""
    with pytest.raises(RuntimeError) as excinfo:
        lp.Runner()
        assert excinfo.value == "Runner is a base class and cannot be initialized"

    with pytest.raises(RuntimeError) as excinfo:
        lp.Runner(config_keys=None)
        assert excinfo.value == "Runner is a base class and cannot be initialized"


def test_runner_config_keymap_updates():
    """This test checks that the keymap is updated when a config_keymap is passed
    to the Runner class."""
    test_keys = {"STAR_SED": "help1", "LIMITS_MAPP_CUT": "help2"}
    config_file_path = os.path.join(TESTDATADIR, "examples/COSMOS.para")
    config_keymap = {
        "STAR_SED": keyword("STAR_SED", "foo"),
        "LIMITS_MAPP_CUT": keyword("LIMITS_MAPP_CUT", "42"),
    }
    runner = lp.Runner(config_keys=test_keys, config_file=config_file_path, config_keymap=config_keymap)
    resulting_keymap = runner.keymap
    assert resulting_keymap["STAR_SED"].value == "foo"
    assert resulting_keymap["LIMITS_MAPP_CUT"].value == "42"


def test_runner_cannot_run():
    """Since the runner class is an abstract class, we expect that it cannot call
    the run method directly."""

    test_keys = {"key1": "help1", "key2": "help2", "key3": "help3"}
    config_file_path = os.path.join(TESTDATADIR, "examples/COSMOS.para")
    runner = lp.Runner(config_keys=test_keys, config_file=config_file_path)

    with pytest.raises(Exception) as excinfo:
        runner.run()
        assert excinfo.value == "runner.py is an abstract class"


def test_runner_verbosity():
    """Check to make sure that verbosity is set correctly via the config_keymap"""
    test_keys = {"key1": "help2", "key2": "help2", "key3": "help3"}
    config_file_path = os.path.join(TESTDATADIR, "examples/COSMOS.para")
    config_keymap = {}
    runner = lp.Runner(config_keys=test_keys, config_file=config_file_path, config_keymap=config_keymap)
    # VERBOSE is set to NO in COSMOS.para
    assert not runner.verbose

    config_keymap = {"VERBOSE": keyword("VERBOSE", "NO")}
    runner = lp.Runner(config_keys=test_keys, config_file=config_file_path, config_keymap=config_keymap)

    assert not runner.verbose

    config_keymap = {"VERBOSE": keyword("VERBOSE", "YES")}
    runner = lp.Runner(config_keys=test_keys, config_file=config_file_path, config_keymap=config_keymap)

    assert runner.verbose


def test_runner_config_file_not_found():
    """Pass in a config file that does not exist and expect a RuntimeError."""

    test_keys = ["key1", "key2", "key3"]
    config_file_path = os.path.join(TESTDATADIR, "foo/bar.para")

    with pytest.raises(RuntimeError) as excinfo:
        _ = lp.Runner(config_keys=test_keys, config_file=config_file_path)
        assert excinfo.value == f"File {config_file_path} not found"


def test_command_line_argument_parsing_basic(monkeypatch):
    """Check to make sure that command line arguments are parsed correctly."""
    test_keys = {"key1": "help1", "key2": "help2", "key3": "help3"}
    monkeypatch.setattr("sys.argv", ["runner.py", "--key1", "foo", "--key2", "42"])
    runner = lp.Runner(config_keys=test_keys)

    assert runner.keymap["key1"].value == "foo"
    assert runner.keymap["key2"].value == "42"
    assert runner.keymap["key3"].value == ""
    assert len(runner.keymap) == 3
    assert runner.config == ""
    assert runner.timer is False
    assert runner.verbose is False

    monkeypatch.setattr("sys.argv", ["runner.py", "--timer"])
    runner = lp.Runner(config_keys=test_keys)
    assert runner.timer is True
    runner.run()
    runner.end()
    assert runner.stop - runner.start > 0


def test_kwargs_arguments():
    test_keys = {"key1": "help1", "key2": "help2", "key3": "help3"}
    runner = lp.Runner(config_keys=test_keys)
    assert [k in runner.keymap for k in test_keys]

    runner = lp.Runner(config_keys={"A": "help"}, a="dummy2")
    assert runner.keymap["A"].value == "dummy2"

    with pytest.raises(RuntimeError) as excinfo:
        runner = lp.Runner(config_keys={"key1": "help"}, key2="unauthorized key")
        assert excinfo.value == f"key2 is not a recognized argument of {runner.__class__.__name__}."


def test_command_line_argument_parsing_with_known_args(monkeypatch):
    """Check to make sure that command line arguments are parsed correctly when
    including known arguments."""
    test_keys = {"key1": "help1", "key2": "help2", "key3": "help3", "QSO_FSCALE": "help"}
    config_file_path = os.path.join(TESTDATADIR, "examples/COSMOS.para")
    monkeypatch.setattr(
        "sys.argv", ["runner.py", "--key1", "foo", "--key2", "42", "--config", config_file_path, "--timer"]
    )
    runner = lp.Runner(config_keys=test_keys)

    assert runner.keymap["key1"].value == "foo"
    assert runner.keymap["key2"].value == "42"
    assert runner.keymap["key3"].value == ""
    assert runner.keymap["QSO_FSCALE"].value == "1."
    assert len(runner.keymap) > 3
    assert runner.config == config_file_path
    assert runner.timer is True
    assert runner.verbose is False
    with pytest.raises(AttributeError) as excinfo:
        _ = runner.args.typ
        assert excinfo.value == "'Runner' object has no attribute 'typ'"


def test_command_line_argument_parsing_with_subclass(monkeypatch):
    """Want to cover the case where the `typ` argument is passed to the runner."""

    class Sedtolib(lp.Runner):
        @property
        def __class__(self):
            return type("Sedtolib", (object,), {})

    test_keys = {"key1": "help1", "key2": "help2", "key3": "help3", "QSO_FSCALE": "help", "typ": "help"}
    config_file_path = os.path.join(TESTDATADIR, "examples/COSMOS.para")
    monkeypatch.setattr(
        "sys.argv",
        [
            "runner.py",
            "--typ",
            "BAR",
            "--key1",
            "foo",
            "--key2",
            "42",
            "--config",
            config_file_path,
            "--timer",
        ],
    )
    runner = Sedtolib(config_keys=test_keys)

    assert runner.keymap["key1"].value == "foo"
    assert runner.keymap["key2"].value == "42"
    assert runner.keymap["key3"].value == ""
    assert runner.keymap["QSO_FSCALE"].value == "1."
    assert len(runner.keymap) > 3
    assert runner.config == config_file_path
    assert runner.timer is True
    assert runner.verbose is False
    assert runner.typ == "BAR"



# ruff: noqa: E402
# ruff: noqa: F403

# Why is this global?
global LEPHAREDIR

from .data_manager import DataManager

dm = DataManager()
dm.configure_directories()  # noqa: F405
LEPHAREDIR = dm.LEPHAREDIR

from ._lephare import *
from ._version import *

# make LEPHAREDIR and LEPHAREWORK avaliable to the C++ codes
get_lephare_env()  # noqa: F405

from ._flt import *
from ._pdf import *
from ._photoz import *
from ._plot_utils import *
from ._spec import *
from .data_retrieval import *
from .default_cosmos_config import *
from .filter import *
from .filter_extinc import *
from .filterSvc import *
from .mag_gal import *
from .magSvc import *
from .prepare import *
from .process import *
from .runner import *
from .sedtolib import *
from .zphota import *



import os
import shutil

import lephare as lp
import numpy as np
from astropy.table import Table


def test_process(test_data_dir: str):
    test_dir = os.path.abspath(os.path.dirname(__file__))
    os.environ["LEPHAREDIR"] = os.path.join(test_dir, "../data")
    os.environ["LEPHAREWORK"] = os.path.join(test_dir, "../tmp")
    # Read the config file.
    config_file = os.path.join(test_data_dir, "examples/COSMOS.para")
    config = lp.read_config(config_file)
    # Run preparation tasks.
    lp.prepare(config)
    # Read the test input catalogue
    input_file = os.path.join(test_data_dir, "examples/COSMOS_first100specz.fits")
    input = Table.read(input_file)
    test_string = "te s"  # Test with spaces
    input["string_input"][0] = test_string
    # Make a reduced column set for the minimal test
    reduced_cols = []
    for c in input.colnames:
        if not c.startswith("f"):
            reduced_cols.append(c)
        elif "IB527" in c:
            reduced_cols.append(c)
        elif "IB679" in c:
            reduced_cols.append(c)
    output, photozlist = lp.process(config, input[reduced_cols], write_outputs=False)
    # Check one of the outputs (results are terrible with just one filter and sparse z grid)
    assert np.isclose(output["Z_BEST"][0], 3.5877994546919934)
    assert len(photozlist[0].pdfmap[11].xaxis) == 51
    pdfs = np.array([photozlist[i].pdfmap[11].vPDF for i in np.arange(len(photozlist))])
    assert np.isclose(np.sum(pdfs), 1001.2774052829275)
    assert output["STRING_INPUT"][0] == test_string
    # Check AUTO_ADAPT
    config["AUTO_ADAPT"] = "YES"
    output, photozlist = lp.process(config, input[reduced_cols], write_outputs=True)

    assert ~np.isclose(output["Z_BEST"][0], 3.5877994546919934)
    assert os.path.isfile("zphot.out")
    assert output["IDENT"][0] == str(input["id"][0])

    a0 = lp.calculate_offsets_from_input(config, input[reduced_cols])
    assert len(a0) == 2

    # Test table formatting
    id, flux, flux_err, context, zspec, string_data = lp.table_to_data(
        config, input[reduced_cols], col_names=reduced_cols, standard_names=False
    )
    assert len(zspec) == 100
    config["FILTER_LIST"] = "cosmos/IB527.lowres,cosmos/IB679.lowres"
    config = lp.all_types_to_keymap(config)
    id, flux, flux_err, context, zspec, string_data = lp.table_to_data(
        config, input[reduced_cols], standard_names=True
    )
    assert len(zspec) == 100


def test_load_sed_list(test_data_dir):
    test_dir = os.path.abspath(os.path.dirname(__file__))
    # Move one of the example sed folders
    _ = shutil.copytree(
        os.path.join(test_dir, "../data/sed/QSO"), os.path.join(test_dir, "../tmp/seds"), dirs_exist_ok=True
    )
    lp.load_sed_list(os.path.join(test_dir, "../tmp/seds/ONE_SED.list"), "QSO")
    # Check the list is there
    assert os.path.exists(os.path.join(test_dir, "../data/sed/QSO/ONE_SED/ONE_SED.list"))
    # Check the sed is there
    assert os.path.exists(os.path.join(test_dir, "../data/sed/QSO/ONE_SED/o5v.sed.ext"))

    # Check it can run even if the file is already there
    lp.load_sed_list(os.path.join(test_dir, "../tmp/seds/ONE_SED.list"), "QSO")
    # Check absolute paths
    with open(os.path.join(test_dir, "../tmp/seds/ONE_SED_ABS.list"), "w") as file:
        file.write(os.path.join(test_dir, "../tmp/seds/o5v.sed.ext"))
    lp.load_sed_list(os.path.join(test_dir, "../tmp/seds/ONE_SED_ABS.list"), "QSO", absolute_paths=True)
    # Clear the copied folders
    shutil.rmtree(os.path.join(test_dir, "../tmp/seds"))
    shutil.rmtree(os.path.join(test_dir, "../data/sed/QSO/ONE_SED"))



import inspect

import lephare as lp
import numpy as np
from astropy.table import Table


def test_all_plots():
    """Very simple test that runs all plots on dummy data to check they run without error."""
    n_rows = 100
    n_filters = 6  # u, g, r, z, J, K
    low, high = 0, 30
    n_z = 100

    # Create dictionary for table columns with fake data
    data = {
        "IDENT": np.arange(n_rows),  # ID as integers
        "Z_BEST": np.random.uniform(low, high, n_rows),
        "Z_BEST68_LOW": np.random.uniform(low, high, n_rows),
        "Z_BEST68_HIGH": np.random.uniform(low, high, n_rows),
        "Z_MED": np.random.uniform(low, high, n_rows),
        "Z_MED68_LOW": np.random.uniform(low, high, n_rows),
        "Z_MED68_HIGH": np.random.uniform(low, high, n_rows),
        "CHI_BEST": np.random.uniform(low, high, n_rows),
        "MOD_BEST": np.random.uniform(low, high, n_rows),
        "EXTLAW_BEST": np.random.uniform(low, high, n_rows),
        "EBV_BEST": np.random.uniform(low, high, n_rows),
        "Z_SEC": np.random.uniform(low, high, n_rows),
        "CHI_SEC": np.random.uniform(low, high, n_rows),
        "MOD_SEC": np.random.uniform(low, high, n_rows),
        "EBV_SEC": np.random.uniform(low, high, n_rows),
        "ZQ_BEST": np.random.uniform(low, high, n_rows),
        "CHI_QSO": np.random.uniform(low, high, n_rows),
        "MOD_QSO": np.random.uniform(low, high, n_rows),
        "MOD_STAR": np.random.uniform(low, high, n_rows),
        "CHI_STAR": np.random.uniform(low, high, n_rows),
        "MAG_OBS()": np.random.uniform(low, high, (n_rows, n_filters)),
        "MAG_ABS()": np.random.uniform(low, high, (n_rows, n_filters)),
        "SCALE_BEST": np.random.uniform(low, high, n_rows),
        "NBAND_USED": np.random.randint(1, n_filters + 1, n_rows),
        "CONTEXT": np.random.uniform(low, high, n_rows),
        "ZSPEC": np.random.uniform(low, high, n_rows),
        "AGE_BEST": np.random.uniform(low, high, n_rows),
        "AGE_INF": np.random.uniform(low, high, n_rows),
        "AGE_MED": np.random.uniform(low, high, n_rows),
        "AGE_SUP": np.random.uniform(low, high, n_rows),
        "LDUST_BEST": np.random.uniform(low, high, n_rows),
        "LDUST_INF": np.random.uniform(low, high, n_rows),
        "LDUST_MED": np.random.uniform(low, high, n_rows),
        "LDUST_SUP": np.random.uniform(low, high, n_rows),
        "LUM_TIR_BEST": np.random.uniform(low, high, n_rows),
        "LUM_TIR_INF": np.random.uniform(low, high, n_rows),
        "LUM_TIR_MED": np.random.uniform(low, high, n_rows),
        "LUM_TIR_SUP": np.random.uniform(low, high, n_rows),
        "MASS_BEST": np.random.uniform(low, high, n_rows),
        "MASS_INF": np.random.uniform(low, high, n_rows),
        "MASS_MED": np.random.uniform(low, high, n_rows),
        "MASS_SUP": np.random.uniform(low, high, n_rows),
        "SFR_BEST": np.random.uniform(low, high, n_rows),
        "SFR_INF": np.random.uniform(low, high, n_rows),
        "SFR_MED": np.random.uniform(low, high, n_rows),
        "SFR_SUP": np.random.uniform(low, high, n_rows),
        "SSFR_BEST": np.random.uniform(low, high, n_rows),
        "SSFR_INF": np.random.uniform(low, high, n_rows),
        "SSFR_MED": np.random.uniform(low, high, n_rows),
        "SSFR_SUP": np.random.uniform(low, high, n_rows),
        "LUM_NUV_BEST": np.random.uniform(low, high, n_rows),
        "LUM_R_BEST": np.random.uniform(low, high, n_rows),
        "LUM_K_BEST": np.random.uniform(low, high, n_rows),
        "PDF_BAY_ZG()": np.random.uniform(low, high, (n_rows, n_z)),
    }

    # Create Astropy Table
    t = Table(data)

    test_utils = lp.PlotUtils(
        t,
        sel_filt=3,
        pos_filt=[0, 1, 2, 4, 5, 5],
        range_z=[0, 0.5, 1, 1.5, 3],
        range_mag=[19, 20.5, 21.5, 22.5, 25],
    )

    # Loop over all methods of the instance
    for name, method in inspect.getmembers(test_utils, predicate=inspect.ismethod):
        # Skip private/internal methods (those starting with '_')
        if not name.startswith("_"):
            print(f"Running method: {name}")
            try:
                # Call the method
                method()
            except TypeError as e:
                # Handle methods that require arguments
                print(f"Skipping {name}, requires arguments: {e}")



import argparse
import os
import sys
import time
from contextlib import suppress

from ._lephare import get_lephare_env, keyword

__all__ = [
    "Runner",
]


class Runner:
    """

    Runner is the base class of the classes Filter, Sedtolib, MagGal, and Zphota,
    responsible to drive the execution of the filter, sedtolib, mag_gal, and
    `zphota` scripts, respectively.

    Configuration of the classes uses either a configuration file, a dictionary of key names
    and corresponding keyword objects, or set of key/values passed as arguments to the constructor

    The order of precedence is : key/values pairs overwrite config_keymap entries, which
    themselves overwrite config_file entries.

    Arguments
    ----------
         config_keys : `list` or `None`, optional
             List of all admissible configuration keys, provided by the inheriting classes
         config_file : `string` or `None`, optional
             Path to a configuration file in LePHARE .para format
         config_keymap : `dict` or `None`, optional
             Dictionary of configuration values provided as 'key/keyword object' pairs.
    """

    def __init__(self, config_keys=None, config_file=None, config_keymap=None, **kwargs):
        # set the LEPHAREDIR and LEPHAREWORK env variable
        get_lephare_env()
        self.keymap = {}
        self.config = ""
        self.verbose = False
        self.typ = None
        self.timer = False

        # Check that the relevant keyword names are defined
        if config_keys is None:
            raise RuntimeError("Runner is a base class and cannot be initialized")
        self.config_keys = config_keys

        # a config_file is passed as argument to the Python constructor
        # note : VERBOSE can be in the config_file, but note type
        if config_file is not None:
            self.keymap = self.parse_config_file(config_file)

        if config_keymap is not None:
            # check validity of config_keymap entries
            self.validate_config_dict(config_keymap)
            # if config_file is not provided but config_keympa is, then use
            # config_keymap. If both are provided, merge them, with
            # config_keymap entries overriding in case of duplicates
            self.keymap = self.keymap | config_keymap
            if "VERBOSE" in self.keymap:
                self.verbose = bool(self.keymap["VERBOSE"].value == "YES")

        # Finally, take the directly provided arguments, which supersede
        # both config_file and config_keymap inputs
        self.validate_config_dict(kwargs, no_raise=False)
        for k, v in kwargs.items():
            uk = k.upper()
            if uk == "TYP":
                self.typ = v.upper()
            # allow verbose to be passed as a bool arg, more pythonesque
            if uk == "VERBOSE":
                if v.__class__ is bool:
                    self.verbose = v
                    kwargs[k] = "YES" if v else "NO"
                else:
                    self.verbose = bool(v.upper() == "YES")

            self.keymap[uk] = keyword(uk, str(kwargs[k]))

        # If self.keymap is still empty, it either means that
        # 1/ an instance of Runner or its inherited classes was called from an executable,
        # 2/ or that it was instantiated in a python session without argument
        if self.keymap == {} and not hasattr(sys, "ps1"):
            self.args = self.config_parser()

        self.update_help()
        return

    # This function takes the config file, read it line per linem and output a keyword map
    def parse_config_file(self, filename):
        """Load config file and set config values.

        Parameters
        ----------
        filename : `string`
            Path to config file
        """
        if not os.path.exists(filename):
            raise RuntimeError("File %s not found" % filename)
        self.config = filename
        keymap = {}

        def config_reader(filename):
            for row in open(filename, "r"):  # noqa: SIM115
                yield row

        config = config_reader(filename)
        for line in config:
            if line[0] != "#" and line != "\n" and not line.isspace():
                splits = line.split()
                splits[0].lstrip()  # remove leading spaces
                if splits[0] != "#" and splits[0] in self.config_keys:
                    try:
                        keymap[splits[0]] = keyword(splits[0], splits[1])
                    except:  # noqa: E722
                        keymap[splits[0]] = keyword(splits[0], "")

                    if splits[0] in self.config_keys and hasattr(self, "parser"):  # pragma no cover
                        self.parser.set_defaults(
                            **{
                                splits[0]: splits[1],
                            }
                        )
        return keymap

    def config_parser(self):
        """Create command line config parser from list of keys"""

        self.parser = argparse.ArgumentParser()
        # these two arguments are not part of the config keys: config is provided directly in python,
        # and timer is dealt with differently in a python ecosystem.
        self.parser.add_argument("--timer", help="switch timer on to time execution", action="store_true")
        self.parser.add_argument("-c", "--config", type=str, default="", help="Path to config file.")
        # add config keys as authorized command line args:
        self.add_authorized_keys()
        args, unknown = self.parser.parse_known_args()

        keymap = {}
        if args.config != "":
            keymap = self.parse_config_file(args.config)

        # capture specific args if they is passed as script argument
        with suppress(Exception):
            self.typ = args.typ

        with suppress(Exception):
            self.timer = args.timer

        # copy the args keywords back into the keymap,
        # setting value to "" for verbose, timer and the like
        for key in self.config_keys:
            if key == "verbose":
                self.verbose = args.verbose
                keymap[key] = keyword(key, "YES") if self.verbose else keyword(key, "NO")
                continue
            try:
                keymap[key] = keyword(key, getattr(args, key))
            except:  # noqa: E722
                if key not in keymap:
                    keymap[key] = keyword(key, "")

        self.keymap = keymap
        return args

    def run(self, **kwargs):
        if self.timer:
            self.start = time.time()

        # update keymap and verbosity based on call arguments
        # this is only when the code is called from python session
        v = kwargs.pop("verbose", self.verbose)
        if v.__class__ is bool:
            self.verbose = v
            self.keymap["VERBOSE"] = keyword("VERBOSE", "YES") if v else keyword("VERBOSE", "NO")
        if v.__class__ is str:
            if v.upper() == "YES":
                self.keymap["VERBOSE"] = keyword("VERBOSE", "YES")
                self.verbose = True
            elif v.upper() == "NO":
                self.keymap["VERBOSE"] = keyword("VERBOSE", "NO")
                self.verbose = False
            else:
                raise RuntimeError("verbose is to be set to False/True or YES/yes//NO/no")

        if "typ" in kwargs:
            self.typ = kwargs.pop("typ", self.typ)
            self.typ = self.typ.upper()
        for k, v in kwargs.items():
            if k.upper() in self.config_keys:
                self.keymap[k.upper()] = keyword(k.upper(), str(v))

    def end(self):
        self.stop = time.time()
        if self.timer:
            print("execution time: %.4g" % (self.stop - self.start))

    def add_authorized_keys(self):
        """Add authorized keys in config keys to the parser"""
        for k, v in self.config_keys.items():
            if k == "typ":
                self.parser.add_argument("-t", "--typ", type=str, default="", help=v, required=True)
            elif k == "verbose":
                self.parser.add_argument("--verbose", help="increase onscreen verbosity", action="store_true")
            else:
                self.parser.add_argument("--%s" % k, type=str, metavar="", help=v)

    def validate_config_dict(self, input_dict, no_raise=True):
        """Check that the input dictionary match the licit arguments"""
        for key in input_dict.copy():
            if key.upper() not in self.config_keys:
                if no_raise:
                    pass
                else:
                    raise RuntimeError(f"{key} is not a recognized argument of {self.__class__.__name__}.")

    def update_help(self):
        """Method to be overloaded by ineriting classes"""
        doc = "Runner is a base class that needs to be inherited from"
        with suppress(Exception):
            self.parser.usage = doc
        self.__doc__ = doc


if __name__ == "__main__":
    raise TypeError("Runner is a base class, not an executable")  # pragma: no cover



import lephare as lp

__all__ = ["default_cosmos_config"]

default_cosmos_config = {
    "ADAPT_BAND": "5",
    "ADAPT_CONTEXT": "-1",
    "ADAPT_LIM": "1.5,23.0",
    "ADAPT_MODBIN": "1,1000",
    "ADAPT_ZBIN": "0.01,6",
    "ADDITIONAL_MAG": "none",
    "ADD_DUSTEM": "NO",
    "ADD_EMLINES": "0,10000",
    "AGE_RANGE": "0.,15.e9",
    "AUTO_ADAPT": "NO",
    "CAT_FMT": "MEME",
    "CAT_IN": "change_me_to_output_filename_required.ascii",
    "CAT_LINES": "0,1000000000",
    "CAT_MAG": "AB",
    "CAT_OUT": "zphot.out",
    "CAT_TYPE": "LONG",
    "CHI2_OUT": "NO",
    "COSMOLOGY": "70,0.3,0.7",
    "DZ_WIN": "1.0",
    "EBV_RANGE": "0,9",
    "EB_V": "0.,0.05,0.1,0.15,0.2,0.25,0.3,0.35,0.4,0.5",
    "EM_DISPERSION": "0.5,0.75,1.,1.5,2.",
    "EM_LINES": "EMP_UV",
    "ERR_FACTOR": "1.5",
    "ERR_SCALE": "0.02,0.02,0.02,0.02,0.02,0.02,0.02,0.05,0.05,0.05,0.05,0.02,0.02,0.02,0.02,0.02,0.02,0.02,0.02,0.02,0.02,0.02,0.02,0.05,0.05,0.05,0.05,0.1,0.2,0.3",  # noqa: E501
    "EXTERNALZ_FILE": "NONE",
    "EXTINC_LAW": "SMC_prevot.dat,SB_calzetti.dat,SB_calzetti_bump1.dat,SB_calzetti_bump2.dat",
    "FILTER_CALIB": "0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1",
    "FILTER_FILE": "filter_cosmos",
    "FILTER_LIST": "cosmos/u_cfht.lowres,cosmos/u_new.pb,hsc/gHSC.pb,hsc/rHSC.pb,hsc/iHSC.pb,hsc/zHSC.pb,hsc/yHSC.pb,vista/Y.lowres,vista/J.lowres,vista/H.lowres,vista/K.lowres,cosmos/IB427.lowres,cosmos/IB464.lowres,cosmos/IB484.lowres,cosmos/IB505.lowres,cosmos/IB527.lowres,cosmos/IB574.lowres,cosmos/IB624.lowres,cosmos/IB679.lowres,cosmos/IB709.lowres,cosmos/IB738.lowres,cosmos/IB767.lowres,cosmos/IB827.lowres,cosmos/NB711.lowres,cosmos/NB816.lowres,vista/NB118.lowres,cosmos/irac_ch1.lowres,cosmos/irac_ch2.lowres,cosmos/irac_ch3.lowres,cosmos/irac_ch4.lowres",  # noqa: E501
    "FILTER_REP": f"{lp.LEPHAREDIR}/filt",
    "FIR_CONT": "-1",
    "FIR_FREESCALE": "YES",
    "FIR_LIB": "NONE",
    "FIR_LMIN": "7.0",
    "FIR_SCALE": "-1",
    "FIR_SUBSTELLAR": "NO",
    "FORB_CONTEXT": "-1",
    "GAL_FSCALE": "1.",
    "GAL_LIB": "LIB_CE",
    "GAL_LIB_IN": "LIB_CE",
    "GAL_LIB_OUT": "CE_COSMOS",
    "GAL_SED": f"{lp.LEPHAREDIR}/sed/GAL/COSMOS_SED/COSMOS_MOD.list",
    "GLB_CONTEXT": "0",
    "INP_TYPE": "F",
    "LIB_ASCII": "NO",
    "LIMITS_MAPP_CUT": "90",
    "LIMITS_MAPP_REF": "1",
    "LIMITS_MAPP_SEL": "1",
    "LIMITS_ZBIN": "0,99",
    "MABS_CONTEXT": "33556478",
    "MABS_FILT": "1,2,3,4",
    "MABS_METHOD": "1",
    "MABS_REF": "11",
    "MABS_ZBIN": "0,0.5,2,4,6",
    "MAGTYPE": "AB",
    "MAG_ABS": "-24,-5",
    "MAG_ABS_QSO": "-30,-10",
    "MAG_REF": "3",
    "MIN_THRES": "0.02",
    "MOD_EXTINC": "0,0",
    "PARA_OUT": f"{lp.LEPHAREDIR}/examples/output.para",
    "PDZ_OUT": "test",
    "PDZ_TYPE": "BAY_ZG",
    "QSO_FSCALE": "1.",
    "QSO_LIB": "LIB_QSO",
    "QSO_LIB_IN": "LIB_QSO",
    "QSO_LIB_OUT": "QSO_COSMOS",
    "QSO_SED": f"{lp.LEPHAREDIR}/sed/QSO/SALVATO09/AGN_MOD.list",
    "RF_COLORS": "32,4,4,13",
    "RM_DISCREPANT_BD": "500",
    "SPEC_OUT": "NO",
    "STAR_FSCALE": "3.432E-09",
    "STAR_LIB": "LIB_STAR",
    "STAR_LIB_IN": "LIB_STAR",
    "STAR_LIB_OUT": "STAR_COSMOS",
    "STAR_SED": f"{lp.LEPHAREDIR}/sed/STAR/STAR_MOD_ALL.list",
    "TRANS_TYPE": "1",
    "VERBOSE": "NO",
    "ZFIX": "NO",
    "ZGRID_TYPE": "0",
    "ZPHOTLIB": "CE_COSMOS,STAR_COSMOS,QSO_COSMOS",
    "Z_INTERP": "YES",
    "Z_METHOD": "BEST",
    "Z_RANGE": "0.,99.99",
    "Z_STEP": "0.01,0.,7.",
}



import matplotlib.pyplot as plt
import numpy as np

__all__ = [
    "plotspec",
]


def plotspec(filename):
    ### Open .spec file[s] and read the parameters
    fsp = open(filename, "r")  # noqa: SIM115
    print("File:", filename)

    bid = fsp.readline()  # header1
    line = fsp.readline()
    line = line.split()
    id = line[0]
    # zspec = line[1]
    # zphot = float(line[2])
    # z68low=float(line[3]); z68hig=float(line[4])

    bid = fsp.readline()  # header2
    line = fsp.readline()
    line = line.split()
    nfilt = int(line[1])

    bid = fsp.readline()  # header3
    line = fsp.readline()
    line = line.split()
    npdf = int(line[1])

    bid = fsp.readline()
    # header4:  Type Nline Model Library Nband    Zphot Zinf Zsup Chi2  PDF
    #     Extlaw EB-V Lir Age  Mass SFR SSFR
    models_info = []
    for _ in range(6):
        line = fsp.readline()
        model = line.split()
        models_info.append(model)

    # Read observed mag, err_mag, filters' lambda_eff and width, models' mag
    mag = np.zeros(nfilt)
    em = np.zeros(nfilt)
    lf = np.zeros(nfilt)
    dlf = np.zeros(nfilt)
    mmod = np.zeros(nfilt)
    mfir = np.zeros(nfilt)
    mphys = np.zeros(nfilt)
    for i in range(nfilt):
        line = fsp.readline()
        line = line.split()
        mag[i] = float(line[0])
        em[i] = float(line[1])
        lf[i] = float(line[2])
        dlf[i] = float(line[3])
        mmod[i] = float(line[4])
        mfir[i] = float(line[5])
        mphys[i] = float(line[6])
        mmod[i] = float(line[8])

    # convert mag(AB syst.) in log(flux)
    ibad = np.where((mmod <= 0) | (mmod > 45))
    mmod = -0.4 * (mmod - 23.91)  # uJy
    mmod[ibad] = -10.0
    ibad = np.where((mphys <= 0) | (mphys > 45))
    mphys = -0.4 * (mphys - 23.91)  # uJy
    mphys[ibad] = -10.0
    ibad = np.where((mfir <= 0) | (mfir > 45))
    mfir = -0.4 * (mfir - 23.91)  # uJy
    mfir[ibad] = -10.0

    zpdf = np.zeros([3, npdf])
    for i in range(npdf):
        line = fsp.readline()
        zpdf[:, i] = np.array(line.split())

    # Read spectra [lambda(A), Mag(AB)]
    # convert in log(F_nu) = -0.4*(mab-23.91) [uJy]
    # Loop over the 6 models (GAL-1, GAL-2, GAL-FIR, GAL-STOCH, QSO, STAR)
    lg = []
    mg = []
    for m in range(6):
        nline = int(models_info[m][1])
        bid = np.zeros([2, nline])
        if nline > 0:
            for i in range(nline):
                line = fsp.readline()
                bid[:, i] = np.array(line.split())
                if bid[1, i] > 35:
                    bid[1, i] = -10.0
                else:
                    bid[1, i] = -0.4 * (bid[1, i] - 23.91)
        lg.append(bid[0, :] / 10000.0)
        mg.append(bid[1, :])

    fsp.close()

    ##############  PLOT  ###############

    ### Initialise the figure
    fig = plt.figure()

    ### Main panel
    ax1 = fig.add_axes(
        [0.1, 0.1, 0.78, 0.78],
        xscale="log",
        xlabel=r"$\lambda$ [$\mu$m]",
        ylabel="log(F$_{\\nu}$) [$\\mu$Jy]",
    )

    # only the reliable obs mag will be plotted:
    em = em * 2.0
    dlf = dlf / 2.0
    mag1 = mag[(mag > 0.0) & (mag < 35) & (em > -3) & (dlf > 50)]
    em1 = em[(mag > 0.0) & (mag < 35) & (em > -3) & (dlf > 50)]
    lf1 = lf[(mag > 0.0) & (mag < 35) & (em > -3) & (dlf > 50)] / 10000.0
    dlf1 = dlf[(mag > 0.0) & (mag < 35) & (em > -3) & (dlf > 50)] / 10000.0

    # Define the limits of the axis, for magnitude having error<10 (if none, try 10)
    if len(mag1[em1 < 1] > 0):
        ymin = max(mag1[em1 < 1] + 3.0)
        ymax = min(mag1[em1 < 1] - 2.0)
    else:
        if len(mag1[em1 < 10] > 0):
            ymin = max(mag1[em1 < 10] + 3.0)
            ymax = min(mag1[em1 < 10] - 2.0)
        else:
            ymin = 15
            ymax = 35
    if ymin > 60:
        ymin = 30

    ic = (em1 >= 0.0) & (em1 < 2.0)
    lf2 = lf1[ic]
    mag2 = -0.4 * (mag1[ic] - 23.91)
    em2 = 0.4 * em1[ic]
    dlf2 = dlf1[ic]
    # low S/N bands:
    ic2 = (em1 >= 2.0) & (em1 < 8.0)
    lf2b = lf1[ic2]
    mag2b = -0.4 * (mag1[ic2] - 23.91)
    em2b = 0.4 * em1[ic2]
    dlf2b = dlf1[ic2]

    # set the plot aspect
    if len(lf1 > 0):
        ax1.axis([min(lf1) * 0.85, max(lf1) * 2, -0.4 * (ymin - 23.91), -0.4 * (ymax - 23.91)])
    else:
        ax1.axis([0, 100000, -0.4 * (ymin - 23.91), -0.4 * (ymax - 23.91)])
    ### plot SED and print info of best-fit models
    col_lst = ["r", "g", "b", "m", "y", "c"]  # each one with a different color
    w_lst = [2, 0.7, 0.7, 1, 1, 0.7]  # different line weight
    a_lst = [1, 0.7, 0.7, 0.7, 0.7, 0.7]  # different alpha
    plt.figtext(
        0.10,
        0.98,
        " Type: (Model Library Nband) z_phot  Chi^2,  Extlaw  EB-V  Lir  Age  logM*  logSFR",
        size="small",
    )
    plt.figtext(0.13, 0.8, "ID=" + id)
    iml = 0
    for im in range(5, -1, -1):
        if int(models_info[im][2]) == -1:
            continue  # print only models really used
        iml = iml + 1  # counter of models used
        ax1.plot(lg[im], mg[im], color=col_lst[im], alpha=a_lst[im], linewidth=w_lst[im])  # plot the SED
        del models_info[im][6:8]  # do not print z_inf and z_sup
        del models_info[im][-1]  # nor sSFR
        info1 = ("  ".join(["%.3f"] * len(models_info[im][5:7]))) % tuple(
            [float(j) for j in models_info[im][5:7]]
        )
        if float(models_info[im][8]) >= 0.0:  # additional information
            info2 = ("   ".join(["%.2f"] * len(models_info[im][8:]))) % tuple(
                [float(j) for j in models_info[im][8:]]
            )
            info2 = ",  " + info2 + "."
        else:
            info2 = "."
        infol = models_info[im][0] + ": (" + " ".join(models_info[im][2:5]) + ")  " + info1 + info2
        plt.figtext(0.15, 0.98 - 0.022 * iml, infol, color=col_lst[im], size="x-small")  # print the rest

    # plot the obs mag...
    ax1.scatter(lf / 10000.0, mmod, color="b", marker="s", s=30)
    ax1.errorbar(lf2b, mag2b, yerr=em2b, xerr=dlf2b, fmt="o", color="0.6")
    ax1.errorbar(lf2, mag2, yerr=em2, xerr=dlf2, fmt="o", color="0.")
    # ... and upper limits
    iu = np.where(em1 < 0)
    if len(iu[0]) > 0:
        lf3 = lf1[iu]
        mag3 = -0.4 * (mag1[iu] - 23.91)
        ax1.quiver(lf3, mag3, 0, -1, units="height", width=0.004, headwidth=5, color="k", pivot="tip")

    ### 2nd panel (inset) showing PDF(z)
    ax2 = fig.add_axes([0.55, 0.17, 0.3, 0.25])
    poscond = np.where((zpdf[1, :] / max(zpdf[1, :]) > 0.002) | (zpdf[2, :] / max(zpdf[2, :]) > 0.002))
    zaxe = zpdf[0, poscond]
    ax2.axis([min(zaxe[0]) - 0.1, max(zaxe[0] + 0.1), 0, 1.1])
    ax2.plot(zpdf[0, :], zpdf[1, :] / max(zpdf[1, :]), color="k", linestyle="solid", label="P(z) Bayesian")
    ax2.plot(zpdf[0, :], zpdf[2, :] / max(zpdf[2, :]), color="k", linestyle="dashed", label="P(z) Profile")
    ax2.legend(fontsize="10")



import subprocess


def test_command_line_tasks():
    """Simply check that they are there and return basic outputs"""
    for task in ["filter", "mag_gal", "sedtolib", "zphota", "filter_extinc"]:
        result = subprocess.run(
            [task, "--help"],
            capture_output=True,
            text=True,
        )
        assert result.returncode == 0
        assert result.stdout.startswith("User defined")



import os

import matplotlib
import numpy as np
import pytest
from lephare import (
    Filter,
    flt,  # noqa: E402
)
from lephare.filterSvc import FilterSvc  # noqa: E402

matplotlib.use("Agg")


@pytest.fixture
def filter_file(test_data_dir):
    return os.path.join(test_data_dir, "filt/subaru/IB527.pb")


def test_flt_class(filter_file):
    tophat = flt(100.0, 200.0, 50)
    assert tophat.name == ""
    # this is wrong but an effect of improper C++ code
    assert tophat.width() == 101.0
    #
    f = flt(-1, filter_file, trans=1, calib=1)
    assert f.width() == pytest.approx(241.9479, 1.0e-4)
    assert f.lambdaMean() == pytest.approx(5262.2831, 1.0e-4)
    # testing relative path
    tests_dir = os.path.abspath(os.path.curdir)
    flt_dir = os.path.dirname(filter_file)
    os.chdir(flt_dir)
    print(os.path.abspath(os.path.curdir))
    f2 = flt(-1, "./IB527.pb", trans=1, calib=1)
    assert f2.width != 0.0
    f3 = flt(-1, "IB527.pb", trans=1, calib=1)
    assert f3.width != 0.0
    os.chdir(tests_dir)


def test_filtersvc(test_data_dir, filter_file):
    f = FilterSvc.from_file(filter_file, trans=1, calib=0)
    assert f.width() == pytest.approx(241.9479, 1.0e-4)
    assert f.lambdaMean() == pytest.approx(5262.2831, 1.0e-4)

    g = FilterSvc.from_svo(-1, "Subaru/Suprime.IB527", "AB")
    if g is not None:
        assert np.allclose(f.data()[1], g.data()[1])
    else:
        print("SVO not tested due to exception raised : server not reachable?")
    # filters in COSMOS.para not available to the unit tests
    fltvec = FilterSvc.from_config(os.path.join(test_data_dir, "examples/COSMOS.para"))
    assert len(fltvec) == 2


def test_filter_base_class(test_data_dir, set_env_vars):
    """Simple test to ensure that we can create an instance of a Filter object."""
    config_file_path = os.path.join(test_data_dir, "examples/COSMOS.para")
    filter = Filter(config_file=config_file_path)
    filter.run()
    assert len(filter.keymap)


def test_filter_with_kwargs(test_data_dir, set_env_vars):
    """Simple test to ensure that we can create an instance of a Filter object
    and that we can pass kwargs when instantiating the object."""
    config_file_path = os.path.join(test_data_dir, "examples/COSMOS.para")
    input_args = {"verbose": True, "TRANS_TYPE": 42}
    filter = Filter(config_file=config_file_path)
    filter.run(**input_args)
    assert len(filter.keymap)
    assert filter.verbose
    assert filter.keymap["TRANS_TYPE"].value == "42"


def test_flt_plot_filter_curve():
    """Simple test that exercises the plot_filter_curve method."""
    tophat = flt(100.0, 200.0, 50)
    tophat.plot_filter_curve()
    tophat.plot_filter_curve(normed=True)

    # this is assertion doesn't mean anything, just here to assert _something_.
    assert tophat.name == ""



import os
from unittest.mock import ANY, MagicMock, patch

import pytest
from lephare import Zphota

TESTDIR = os.path.abspath(os.path.dirname(__file__))
TESTDATADIR = os.path.join(TESTDIR, "../data")


# Test init


def test_with_config_file():
    """Use a config file to instantiate the Zphota class. Expect the keymap to be
    populated."""
    config_file_path = os.path.join(TESTDATADIR, "examples/COSMOS.para")
    zp = Zphota(config_file=config_file_path)
    assert len(zp.keymap)
    assert zp.keymap["CAT_IN"].value == "bidon"


def test_zphota_config_file_not_found():
    """Use a non-existent config file to instantiate the Zphota class. Expect a
    RuntimeError."""
    config_file_path = os.path.join(TESTDATADIR, "foo/bar.para")

    with pytest.raises(RuntimeError) as excinfo:
        _ = Zphota(config_file=config_file_path)
        assert excinfo.value == f"File {config_file_path} not found"


def test_with_command_line_config(monkeypatch):
    """Use command line arguments to instantiate the Zphota class. Expect the keymap
    to be populated."""
    monkeypatch.setattr("sys.argv", ["zphota.py", "--CAT_IN", "cat_foo", "--Z_INTERP", "YES"])
    zp = Zphota()
    assert zp.keymap["CAT_IN"].value == "cat_foo"
    assert zp.keymap["Z_INTERP"].value == "YES"


def test_override_config_file_with_command_line(monkeypatch):
    """Use the command line to feed in a config file, but override one of the values
    with a command line value.

     This is the expected behavior, but only if we pass the config file in on the
    command line. Otherwise, as we test in the last part of this, we do not override
    the value."""
    config_file_path = os.path.join(TESTDATADIR, "examples/COSMOS.para")
    monkeypatch.setattr("sys.argv", ["zphota.py", "--config", config_file_path, "--CAT_IN", "cat_foo"])
    zp = Zphota()
    assert zp.keymap["CAT_IN"].value == "cat_foo"
    assert zp.keymap["Z_INTERP"].value == "YES"


# Test run


def run_configured_zp(zp, config_file_path):
    # Create mock onesource objects
    mock_sources = [
        MagicMock(spec=["spec", "zs"]),
        MagicMock(spec=["spec", "zs"]),
        MagicMock(spec=["spec", "zs"]),
    ]

    # Set the attributes
    mock_sources[0].spec = "spec1"
    mock_sources[0].zs = 0.1
    mock_sources[1].spec = "spec2"
    mock_sources[1].zs = 0.2
    mock_sources[2].spec = "spec3"
    mock_sources[2].zs = 0.3

    # Run zphota with cpp functions mocked
    with patch(
        "lephare._lephare.PhotoZ.read_photoz_sources", return_value=mock_sources
    ) as mock_read_photoz_sources:
        with patch("lephare._lephare.PhotoZ.run_photoz") as mock_run_photoz:
            with patch("lephare._lephare.PhotoZ.write_outputs") as mock_write_outputs:
                mock_run_photoz.return_value = None
                mock_write_outputs.return_value = None

                zp.run()

                mock_read_photoz_sources.assert_called_once()
                mock_run_photoz.assert_called_once()
                mock_run_photoz.assert_called_with(mock_sources, ANY)
                mock_write_outputs.assert_called_once()
                mock_write_outputs.assert_called_with(mock_sources, ANY)

                assert os.path.realpath(zp.keymap["c"].value) == os.path.realpath(config_file_path)


def test_run_zp_config_file(monkeypatch):
    """Use a config file to run the Zphota class (ie, running from a python session).

    Note that we mock the cpp functions that are called by the run method."""
    config_file_path = os.path.join(TESTDATADIR, "examples/COSMOS.para")
    zp = Zphota(config_file=config_file_path)

    run_configured_zp(zp, config_file_path)


def test_run_zp_command_line(monkeypatch):
    """Use the command line to run the Zphota class."""
    config_file_path = os.path.join(TESTDATADIR, "examples/COSMOS.para")
    monkeypatch.setattr("sys.argv", ["zphota.py", "--config", config_file_path, "--timer"])
    zp = Zphota()

    run_configured_zp(zp, config_file_path)


def test_run_zp_extra_command_line_flags(monkeypatch):
    """Run the zphota class with timer and verbose flags."""
    config_file_path = os.path.join(TESTDATADIR, "examples/COSMOS.para")
    monkeypatch.setattr("sys.argv", ["zphota.py", "--config", config_file_path, "--timer", "--verbose"])
    zp = Zphota(config_file=config_file_path)

    run_configured_zp(zp, config_file_path)


def test_run_zp_auto_adapt_yes(monkeypatch):
    """Run the zphota class with AUTO_ADAPT toggled on."""
    config_file_path = os.path.join(TESTDATADIR, "examples/COSMOS.para")
    monkeypatch.setattr("sys.argv", ["zphota.py", "--config", config_file_path, "--AUTO_ADAPT", "YES"])
    zp = Zphota()

    run_configured_zp(zp, config_file_path)


def test_run_zp_auto_adapt_no(monkeypatch):
    """Run the zphota class with AUTO_ADAPT toggled off."""
    config_file_path = os.path.join(TESTDATADIR, "examples/COSMOS.para")
    monkeypatch.setattr("sys.argv", ["zphota.py", "--config", config_file_path, "--AUTO_ADAPT", "NO"])
    zp = Zphota()

    run_configured_zp(zp, config_file_path)



import os
import shutil
import time

import numpy as np

import lephare as lp

__all__ = ["object_types", "process", "table_to_data", "calculate_offsets_from_input", "load_sed_list"]

object_types = ["STAR", "GAL", "QSO"]


def process(
    config,
    input,
    col_names=None,
    standard_names=False,
    filename=None,
    write_outputs=False,
):
    """Run all required steps to produce photometric redshift estimates

    Parameters
    ==========
    config : dict of lephare.keyword
        The configuration for the run
    input : astropy.table.Table
        The input table which must satisfy column name requirements depending
        on other optional inputs.
    col_names : list
        Input catalogue column names. We will use ordering to determine meaning
    standard_names : bool
        If true we assume standard names.
    filename : str
        Output file name for the output catalogue.
    write_outputs : bool
        Whether to write the output spectra, PDF, and ascii file if specified
        in the config. By default these are not written to save space.

    Returns
    =======
    output : astropy.table.Table
        The output table.
    photozlist : list of lephare.onesource
        List of lephare onesource objects.
    """
    # ensure that all values in the keymap are keyword objects
    config = lp.all_types_to_keymap(config)

    id, flux, flux_err, context, zspec, string_data = table_to_data(
        config, input, col_names=col_names, standard_names=standard_names
    )
    ng = len(id)
    n_filters = len(config["FILTER_LIST"].value.split(","))
    print(f"Processing {ng} objects with {n_filters} bands")
    photz = lp.PhotoZ(config)
    if config["AUTO_ADAPT"].split_bool("", 1):
        print("AUTO_ADAPT is set to YES. Computing offsets.")
        # Loop over all ng galaxies!
        srclist = []
        for i in range(ng):
            one_obj = lp.onesource(i, photz.gridz)
            one_obj.readsource(str(id[i]), flux[i], flux_err[i], context[i], zspec[i], str(string_data[i]))
            photz.prep_data(one_obj)
            srclist.append(one_obj)

        # compute the offset, depending on the option in the code (AUTO_ADAPT, or APPLY_SYSSHIFT
        a0 = photz.compute_offsets(srclist)
    else:
        a0 = np.zeros(len(config["FILTER_LIST"].value.split(",")))
    offsets = ",".join(np.array(a0).astype(str))
    offsets = "# Offsets added to the modeled magnitudes (or substracted to the observed): " + offsets + "\n"
    print(offsets)

    # create the onesource objects
    photozlist = []
    for i in range(ng):
        one_obj = lp.onesource(i, photz.gridz)
        one_obj.readsource(str(id[i]), flux[i], flux_err[i], context[i], zspec[i], str(string_data[i]))
        photz.prep_data(one_obj)
        photozlist.append(one_obj)

    # Perform the main run
    photz.run_photoz(photozlist, a0)
    # Write outputs if requested
    if write_outputs:
        photz.write_outputs(photozlist, int(time.time()))
    output = photz.build_output_tables(photozlist, para_out=None, filename=filename)
    # Return the table and all the onesource objects
    return output, photozlist


def calculate_offsets_from_input(config, input, col_names=None, standard_names=False):
    """Calculate the zero point offsets for objects with spectroscopic redshifts

    We want to have this available as an independent method so that it can be
    called as a training stage prior to running at scale.

    Parameters
    ==========
    config : dict of lephare.keyword
        The configuration for the run
    input : astropy.table.Table
        The input table which must satisfy column name requirements depending
        on other optional inputs.
    col_names : list
        Input catalogue column names. We will use ordering to determine meaning
    standard_names : bool
        If true we assume standard names.

    Returns
    =======
    a0 : np.array
        Offsets a0
    """
    config = lp.all_types_to_keymap(config)
    id, flux, flux_err, context, zspec, string_data = table_to_data(
        config, input, col_names=col_names, standard_names=standard_names
    )
    ng = len(id)
    n_filters = len(config["FILTER_LIST"].value.split(","))
    print(f"Processing {ng} objects with {n_filters} bands")
    photz = lp.PhotoZ(config)
    # Loop over all ng galaxies!
    srclist = []
    for i in range(ng):
        one_obj = lp.onesource(i, photz.gridz)
        one_obj.readsource(str(id[i]), flux[i], flux_err[i], context[i], zspec[i], str(string_data[i]))
        photz.prep_data(one_obj)
        srclist.append(one_obj)

    a0 = photz.compute_offsets(srclist)
    offsets = ",".join(np.array(a0).astype(str))
    offsets = "Offsets from auto-adapt: " + offsets + "\n"
    print(offsets)
    return a0


def table_to_data(config, input, col_names=None, standard_names=False):
    """Take an astropy table and return the arrays required for a run.

    We assume that either the columns are in the standard LePHARE order or
    that the user provides the colnames in the standard order or they
    use a standard naming convention and set the standard_names=True

    The column names are id, f_filtername, ferr_filtername (for all filters)
    and context, zspec, string_input.

    Parameters
    ==========
    config : dict of lephare.keyword
        The config keymap. We need this to know if we have magnitudes or
        fluxes and to get the filter names.
    input : astropy.table.Table
        The input catalogue.
    col_names : list of str
        The column names in the default order.
    standard_names : bool
        If this is set we assume a standard column naming convention.

    Returns
    =======
    id : np.array
        The object ids
    flux : np.array
        The object fluxes or magnitudes
    flux_err : np.array
        The flux or magnitude errors
    context : np.array
        The context determining flux usage
    zspec : np.array
        The spectroscopic redshifts.
    string_input : np.array
        Additional notes as a string.
    """
    config = lp.all_types_to_keymap(config)
    n_filters = len(config["FILTER_LIST"].value.split(","))
    if col_names is not None:
        print("Using user defined column names based on ordering.")
        assert len(input.colnames) == 2 * n_filters + 4

    elif standard_names:
        print(
            "Using standard column names id, f_filtername,ferr_filtername,... context, zspec, string_input."
        )
        col_names = ["id"]
        for f in config["FILTER_LIST"].value.split(","):
            col_names += [f"f_{f}"]
            col_names += [f"ferr_{f}"]
        col_names += ["context"]
        col_names += ["zspec"]
        col_names += ["string_input"]
    else:
        print("Using user columns from input table assuming they are in the standard order.")
        assert len(input.colnames) == 2 * n_filters + 4
        col_names = list(np.full(2 * n_filters + 4, ""))
        col_names[0] = input.colnames[0]
        col_names[1 : 2 * n_filters : 2] = input.colnames[1 : 2 * n_filters : 2]
        col_names[2 : 2 * n_filters + 1 : 2] = input.colnames[2 : 2 * n_filters + 1 : 2]
        col_names[-3] = input.colnames[-3]
        col_names[-2] = input.colnames[-2]
        col_names[-1] = input.colnames[-1]
        # print(col_names)

    assert len(col_names) == 2 * n_filters + 4
    id = [str(i) for i in input[col_names[0]]]
    flux = input[col_names[1 : 2 * n_filters : 2]]
    flux = np.array([np.array(flux[c]) for c in flux.colnames])
    flux = flux.T
    flux_err = input[col_names[2 : 2 * n_filters + 1 : 2]]
    flux_err = np.array([np.array(flux_err[c]) for c in flux_err.colnames])
    flux_err = flux_err.T
    context = input[col_names[-3]]
    zspec = input[col_names[-2]]
    string_data = input[col_names[-1]]

    # Perform basic checks on table
    # Replace nans with -99

    mask = np.isnan(flux)
    mask |= np.isnan(flux_err)
    flux[mask] = -99.0
    flux_err[mask] = -99.0
    return id, flux, flux_err, context, zspec, string_data


def load_sed_list(path_to_list, object_type, absolute_paths=False):
    """Take a sed list and move it with the sed files to the work directory.

    We want to be able to load SED lists from other sources. These may use
    absolute paths or relative to the list file. They must adopt the lepahre
    standard which is relative to the sed/$TYPE/ directory

    Parameters
    ==========
    path_to_list : str
        The path to the SED list file. This can be absolute or relative.
    object_type : {'STAR', 'GAL', 'QSO'}
        The type of object. Must be one of "STAR", "GAL", or "QSO".
    absolute_paths : bool
        Set to True if list of absolute paths and not relative to list file

    Returns
    =======
        new_config_val : str
            The location of the new list file with respect to current
            LEPHAREWORK directory. This is the value to set in the config.
    """
    # The type must by one of the three lephare standards
    assert object_type in object_types
    # The original file must exist
    assert os.path.isfile(path_to_list)

    # Set folder and file names.
    name = path_to_list.split("/")[-1].split(".")[0]
    old_dir = os.path.dirname(path_to_list)
    new_dir = f"{lp.dm.lephare_dir}/sed/{object_type}/{name}"
    new_list_file = f"{new_dir}/{name}.list"
    # Create the config value that needs to be passed to lephare.
    new_config_val = f"sed/{object_type}/{name}"
    if os.path.exists(new_list_file):
        print(
            f"List file already exists at {new_list_file}\n"
            "The SEDs may already have been loaded to the work directory."
        )
        return new_config_val

    # Get the list of sed files.
    with open(path_to_list, "r") as f:
        lines = f.readlines()
    sed_files = []
    for line in lines:
        # Ignore commented files
        if line.strip()[0] != "#":
            sed_files.append(line.replace("\n", ""))

    # Make the folder to hold the seds.
    assert not os.path.isdir(new_dir)
    print(
        f"Creating new sed directory at {new_dir}.\n" "Attempting to copy sed files and list files in place."
    )
    os.makedirs(new_dir)

    # Make the new list of files with paths from the type folder
    new_list = ""
    for sed_file in sed_files:
        new_file = f"{sed_file.split('/')[-1]}"
        new_list += f"{name}/{new_file}\n"
        # Copy files
        if absolute_paths:
            shutil.copyfile(sed_file, f"{new_dir}/{new_file}")
        else:
            shutil.copyfile(f"{old_dir}/{sed_file.split('/')[-1]}", f"{new_dir}/{new_file}")

    # Open the list file for writing and add all the seds
    with open(new_list_file, "w") as file:
        # Define the data to be written
        file.write(new_list)
    return new_config_val



import os

import lephare as lp
from astropy.table import Table
from lephare.prepare import all_types_to_keymap


def test_version():
    """Check we have a version."""
    assert isinstance(lp.__version__, str)


def test_prepare(test_data_dir: str):
    test_dir = os.path.abspath(os.path.dirname(__file__))
    os.environ["LEPHAREDIR"] = os.path.join(test_dir, "../data")
    os.environ["LEPHAREWORK"] = os.path.join(test_dir, "../tmp")
    config = lp.read_config(os.path.join(test_data_dir, "examples/COSMOS.para"))
    lp.prepare(config)
    # Check the default config is consistent
    default_config = lp.default_cosmos_config
    assert config["Z_STEP"] != default_config["Z_STEP"]
    assert default_config["FILTER_REP"] == str(os.path.join(test_data_dir, "filt"))
    # Check it made the galaxy binary file
    assert os.path.exists(os.path.join(test_dir, "../tmp/lib_mag/CE_COSMOS.bin"))
    gal_mag = Table.read(os.path.join(test_dir, "../tmp/lib_mag/CE_COSMOS.dat"), format="ascii")
    assert len(gal_mag.colnames) == 14  # Contains additional EM_DISPERSION column


def test_all_types_to_keymap():
    for in_dict in [{"key": "map"}, {"key": lp.keyword("key", "map")}]:
        out_dict = all_types_to_keymap(in_dict)
        for i, o in zip(in_dict, out_dict):
            assert i == o
            assert out_dict[o].__class__ == lp.keyword
            assert out_dict[o].name == "key"
            assert out_dict[o].value == "map"


def test_config_formatting():
    """Some simple tests of configs and switching between formats"""
    test_dir = os.path.abspath(os.path.dirname(__file__))
    os.environ["LEPHAREDIR"] = os.path.join(test_dir, "../data")
    os.environ["LEPHAREWORK"] = os.path.join(test_dir, "../tmp")
    # Make a string dict config and keymap
    config = lp.default_cosmos_config.copy()
    keymap = all_types_to_keymap(config)
    # Check they are consistent
    assert config["FILTER_FILE"] == keymap["FILTER_FILE"].value
    # Check the type of the keymap is a lephare.keyword
    assert type(keymap["Z_STEP"]) == lp.keyword
    lp.write_para_config(config, os.path.join(os.environ["LEPHAREWORK"], "test.para"))
    # Check it made the file
    assert os.path.exists(os.path.join(os.environ["LEPHAREWORK"], "test.para"))
    os.remove(os.path.join(os.environ["LEPHAREWORK"], "test.para"))
    # Make the file again from the keymap instead
    lp.write_para_config(keymap, os.path.join(os.environ["LEPHAREWORK"], "test.para"))
    # Check it also made the file from the keymap
    assert os.path.exists(os.path.join(os.environ["LEPHAREWORK"], "test.para"))
    os.remove(os.path.join(os.environ["LEPHAREWORK"], "test.para"))
    # Check it converts correctly back to a string dict
    assert lp.string_dict_to_keymap(config)["FILTER_FILE"].value == "filter_cosmos"
    # Check the reverse conversion
    assert lp.keymap_to_string_dict(keymap)["FILTER_FILE"] == "filter_cosmos"



import os
import pathlib
import platform
import re
import subprocess
import sys
from distutils.version import LooseVersion

from setuptools import Command, Extension, setup
from setuptools.command.build_ext import build_ext

here = pathlib.Path(__file__).parent.resolve()
# Get the long description from the README file
long_description = (here / "README.md").read_text(encoding="utf-8")


class CMakeExtension(Extension):
    def __init__(self, name, sourcedir=""):
        Extension.__init__(self, name, sources=[])
        self.sourcedir = os.path.abspath(sourcedir)


class CMakeBuild(build_ext):
    def run(self):
        try:
            out = subprocess.check_output(["cmake", "--version"])
        except OSError:
            raise RuntimeError(
                "CMake must be installed to build the following extensions: "
                + ", ".join(e.name for e in self.extensions)
            )

        if platform.system() == "Windows":
            cmake_version = LooseVersion(re.search(r"version\s*([\d.]+)", out.decode()).group(1))
            if cmake_version < "3.1.0":
                raise RuntimeError("CMake >= 3.1.0 is required on Windows")

        for ext in self.extensions:
            self.build_extension(ext)

    def build_extension(self, ext):
        extdir = os.path.abspath(os.path.dirname(self.get_ext_fullpath(ext.name)))
        cmake_args = [
            "-DCMAKE_INSTALL_PREFIX=${CMAKE_SOURCE_DIR}",
            "-DCMAKE_LIBRARY_OUTPUT_DIRECTORY=" + extdir,
            "-DPYTHON_EXECUTABLE=" + sys.executable,
        ]

        cfg = "Debug" if self.debug else "Release"
        build_args = ["--config", cfg]

        if platform.system() == "Windows":
            cmake_args += [f"-DCMAKE_LIBRARY_OUTPUT_DIRECTORY_{cfg.upper()}={extdir}"]
            if sys.maxsize > 2**32:
                cmake_args += ["-A", "x64"]
            build_args += ["--", "/m"]
        else:
            cmake_args += ["-DCMAKE_BUILD_TYPE:STRING=" + cfg]
            build_args += ["--", "-j2"]

        env = os.environ.copy()
        env["CXXFLAGS"] = '{} -DVERSION_INFO=\\"{}\\"'.format(
            env.get("CXXFLAGS", ""), self.distribution.get_version()
        )
        if not os.path.exists(self.build_temp):
            os.makedirs(self.build_temp)
        subprocess.check_call(["cmake", ext.sourcedir] + cmake_args, cwd=self.build_temp, env=env)
        subprocess.check_call(["cmake", "--build", "."] + build_args, cwd=self.build_temp)
        subprocess.check_call(["cmake", "--install", "."], cwd=self.build_temp)
        print()  # Add an empty line for cleaner output


class MakeDoc(Command):
    user_options = []

    def initialize_options(self):
        pass

    def finalize_options(self):
        pass

    def run(self):
        cmd = "doxygen Doxyfile"
        subprocess.check_call(["doxygen", "Doxyfile"], cwd="doc")


setup(
    name="lephare",
    url="https://lephare.readthedocs.io/en/latest/",
    long_description=long_description,
    long_description_content_type="text/markdown",
    # add extension module
    ext_modules=[CMakeExtension("lephare._lephare")],
    include_package_data=True,
    # add custom build_ext command
    cmdclass=dict(build_ext=CMakeBuild, doc=MakeDoc),
    zip_safe=False,
    python_requires=">=3.9",
    extras_require={"test": "pytest"},
)



import os
import tempfile

import numpy as np
from lephare import keyword, read_command, read_config

TESTDIR = os.path.abspath(os.path.dirname(__file__))
LEPHAREDIR = os.path.join(TESTDIR, "..")


def test_keyword_basic():
    # test that default is taken when value is empty
    k = keyword("k1", "")
    assert k.split_string("DEFAULT", 1)[0] == "DEFAULT"
    # check basic assignment
    k = keyword("k2", "1")
    assert k.name == "k2"
    assert k.value == "1"
    # test mismatch in size
    assert np.array_equal(k.split_string("0", 2), ["1", "1"])
    k = keyword("k3", "1,1")
    assert np.array_equal(k.split_string("0", 3), ["0", "0", "0"])


def test_keyword_array():
    k = keyword("k", "1,1")
    assert np.array_equal(k.split_string("0", 2), ["1", "1"])
    assert np.array_equal(k.split_int("0", 2), [1, 1])
    assert np.array_equal(k.split_long("0", 2), [1, 1])
    assert np.array_equal(k.split_bool("0", 2), [True, True])
    assert np.array_equal(k.split_double("0", 2), [1.0, 1.0])


def test_keyword_bool():
    k = keyword("k", "1")
    assert k.split_bool("0", 1)[0]
    k = keyword("k", "YES")
    assert k.split_bool("0", 1)[0]
    k = keyword("k", "yes")
    assert k.split_bool("0", 1)[0]
    k = keyword("k", "True")
    assert k.split_bool("0", 1)[0]
    k = keyword("k", "true")
    assert k.split_bool("0", 1)[0]
    k = keyword("k", "ANYTHING ELSE")
    assert not k.split_bool("0", 1)[0]


def test_keyword_expand_path():
    lepharedir = os.environ["LEPHAREDIR"]
    home = os.environ["HOME"]
    k = keyword("k", "$LEPHAREDIR/unittest")
    assert k.value == os.path.join(lepharedir, "unittest")
    k = keyword("k", "$HOME/unittest")
    assert k.value == os.path.join(home, "unittest")


def test_keyword_read_config(test_data_dir):
    config_file = os.path.join(test_data_dir, "examples/COSMOS.para")
    km = read_config(config_file)
    for k in km:
        assert k[0] != "#"
        assert k[0] != " "
        assert k[0] != "\t"


def test_keyword_read_command():
    km = read_command(["exec", "-c", "config", "--arg1", "val1", "-arg2", "val2"])
    for k in km:
        if k == "c":
            assert km[k].value == "config"
        if k == "arg1":
            assert km[k].value == "val1"
        if k == "arg2":
            assert km[k].value == "val2"


def test_keyword_misformed_kwd():
    # normal
    k = keyword("k", "1,2,3")
    assert np.array_equal(k.split_string("0", -1), ["1", "2", "3"])
    # space
    k = keyword("k", "1,2, 3")
    assert np.array_equal(k.split_string("0", -1), ["1", "2", "3"])
    # tab
    k = keyword("k", "1,2,\t3")
    assert np.array_equal(k.split_string("0", -1), ["1", "2", "3"])

    with tempfile.TemporaryDirectory() as dir_name:
        filename = os.path.join(dir_name, "tmp")
        with open(filename, "w") as f:
            f.writelines(
                [
                    "FILTER_CALIB    0,0,0,0, 0,0,0,0,0,0,0,0,0,0,0,0,0,0,\t0",
                    "\nFILTER_FILE     filter_cosmos",
                ]
            )
        km = read_config(filename)
        assert np.array_equal(km["FILTER_CALIB"].value, "0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0")
        assert np.array_equal(km["FILTER_FILE"].value, "filter_cosmos")



import os
import tempfile
import time
from unittest.mock import patch

import pytest
from lephare import data_manager as dm
from platformdirs import user_cache_dir


def test_data_manager_default_directories(unset_env_vars):
    """Test DataManager default configuration"""
    new_dm = dm.DataManager()
    assert new_dm.LEPHAREDIR is None
    assert new_dm.LEPHAREWORK is None


def test_data_manager_with_predefined_directories(unset_env_vars):
    """Test DataManager with predefined directories"""
    os.environ["LEPHAREDIR"] = "/tmp1"
    os.environ["LEPHAREWORK"] = "/tmp2"
    new_dm = dm.DataManager()
    assert new_dm.LEPHAREDIR == "/tmp1"
    assert new_dm.LEPHAREWORK == "/tmp2"


def test_data_manager_with_predefined_dir_directory(unset_env_vars):
    """Test DataManager with predefined dir directory"""
    os.environ["LEPHAREDIR"] = "/tmp1"
    new_dm = dm.DataManager()
    assert new_dm.LEPHAREDIR == "/tmp1"
    assert new_dm.LEPHAREWORK is None


def test_data_manager_with_predefined_work_directory(unset_env_vars):
    """Test DataManager with predefined work directory"""
    os.environ["LEPHAREWORK"] = "/tmp2"
    new_dm = dm.DataManager()
    assert new_dm.LEPHAREDIR is None
    assert new_dm.LEPHAREWORK == "/tmp2"


def test_data_manager_configure_directories_with_default(unset_env_vars):
    """Test DataManager configure_directories method with default directories"""
    new_dm = dm.DataManager()
    new_dm.configure_directories()
    expected_data_directory = user_cache_dir("lephare", ensure_exists=True) + "/data"
    expected_work_directory = user_cache_dir("lephare", ensure_exists=True) + "/work"
    assert os.getenv("LEPHAREDIR") == expected_data_directory
    assert os.getenv("LEPHAREWORK") == expected_work_directory
    assert expected_data_directory == new_dm.LEPHAREDIR
    assert expected_work_directory == new_dm.LEPHAREWORK


def test_create_new_run_with_unlinked_work_dir():
    """Make sure that a runtime error is raised when we attempt to create a new
    run directory structure within a pre-existing and un-linked work directory"""
    with pytest.raises(RuntimeError):
        with tempfile.TemporaryDirectory() as tmpdir:
            os.environ["LEPHAREWORK"] = tmpdir
            new_dm = dm.DataManager()
            new_dm.create_new_run()


def test_create_new_run_in_mock_cache_dir(unset_env_vars):
    """Make sure that the expected directory structure is created in a mock cache
    directory"""
    with tempfile.TemporaryDirectory() as tmpdir:
        with patch("lephare.data_manager.user_cache_dir", return_value=tmpdir):
            new_dm = dm.DataManager()
            new_dm.configure_directories()
            assert len(os.listdir(os.path.join(tmpdir, "runs"))) == 1


def test_create_new_run_creates_symlink(unset_env_vars):
    """Make sure that the expected symlinked directory structure is created"""
    with tempfile.TemporaryDirectory() as tmpdir:
        with patch("lephare.data_manager.user_cache_dir", return_value=tmpdir):
            # Set up data manager and make a run
            new_dm = dm.DataManager()
            new_dm.configure_directories()
            new_dm.create_new_run()
            # Check work directory exists
            assert os.path.isdir(os.path.join(tmpdir, "work"))
            # Check the symlink exists
            assert os.path.islink(os.path.join(tmpdir, "work"))
            assert os.path.realpath(os.path.join(tmpdir, "work")) == os.path.realpath(
                os.path.join(tmpdir, "runs", os.listdir(os.path.join(tmpdir, "runs"))[0])
            )


def test_create_new_run_overwrites_symlink(unset_env_vars):
    """Make sure that the expected symlinked directory structure is created, and
    the previous symlink is overwritten"""
    with tempfile.TemporaryDirectory() as tmpdir:
        with patch("lephare.data_manager.user_cache_dir", return_value=tmpdir):
            # Set up data manager and make a run
            new_dm = dm.DataManager()
            new_dm.configure_directories()
            new_dm.create_new_run()
            # Sleep for a second to ensure the timestamp is different
            time.sleep(1)
            # Create a new run
            new_dm.create_new_run()
            # Check the symlink has been moved from the original directory to new directory
            sorted_timestamped_dirs = sorted(os.listdir(os.path.join(tmpdir, "runs")))
            assert os.readlink(os.path.join(tmpdir, "work")) != sorted_timestamped_dirs[0]
            assert os.readlink(os.path.join(tmpdir, "work")) != sorted_timestamped_dirs[-1]


def test_create_work_subdirectories():
    """Make sure that the expected subdirectories are created"""
    with tempfile.TemporaryDirectory() as tmpdir:
        new_dm = dm.DataManager()
        new_dm.create_work_subdirectories(tmpdir)
        assert os.path.isdir(os.path.join(tmpdir, "filt"))
        assert os.path.isdir(os.path.join(tmpdir, "lib_bin"))
        assert os.path.isdir(os.path.join(tmpdir, "lib_mag"))
        assert os.path.isdir(os.path.join(tmpdir, "zphota"))
        assert len(os.listdir(tmpdir)) == 4


def test_create_work_subdirectories_with_existing_directories():
    """Makes sure that no exceptions are raised if the directories already exist
    and ensure that existing files are not overwritten or deleted."""
    # Create a temp directory
    with tempfile.TemporaryDirectory() as tmpdir:
        # Create the `filt` directory and add a test file to it
        os.makedirs(os.path.join(tmpdir, "filt"))
        with open(os.path.join(tmpdir, "filt", "test.txt"), "w") as file:
            file.write("test")

        # Create the other directories
        new_dm = dm.DataManager()
        new_dm.create_work_subdirectories(tmpdir)
        assert os.path.isdir(os.path.join(tmpdir, "filt"))
        assert os.path.isdir(os.path.join(tmpdir, "lib_bin"))
        assert os.path.isdir(os.path.join(tmpdir, "lib_mag"))
        assert os.path.isdir(os.path.join(tmpdir, "zphota"))

        # Ensure the test file is still there
        with open(os.path.join(tmpdir, "filt", "test.txt"), "r") as file:
            assert file.read() == "test"


def test_remove_empty_run_directories(unset_env_vars):
    """Make sure that the method to remove empty run directories does remove
    empty directories."""
    with tempfile.TemporaryDirectory() as tmpdir:
        with patch("lephare.data_manager.user_cache_dir", return_value=tmpdir):
            runs_dir = os.path.join(tmpdir, "runs")
            # Set up data manager and make a run
            new_dm = dm.DataManager()
            new_dm.configure_directories()
            new_dm.create_new_run()
            # Assert there's just one run directory
            assert len(os.listdir(runs_dir)) == 1
            # Make more (sleeping to ensure timestamps are different)
            time.sleep(1)
            new_dm.create_new_run()
            time.sleep(1)
            new_dm.create_new_run()
            # Assert there are now three run directories
            assert len(os.listdir(runs_dir)) == 3
            # Remove the empty run directories
            new_dm.remove_empty_run_directories()
            # Assert there's just one run directory (the new one created to preserve symlink)
            assert len(os.listdir(runs_dir)) == 1


def test_remove_empty_skips_nonempty_symlinked_run(unset_env_vars):
    """Make sure that the method to remove empty run directories does not remove
    non-empty directories. This is the case where the non-empty directory is the
    symlinked work directory, so we do not need to generate a new run directory
    to preserve the symlink."""
    with tempfile.TemporaryDirectory() as tmpdir:
        with patch("lephare.data_manager.user_cache_dir", return_value=tmpdir):
            # Set up data manager and make three runs (sleeping for diff timestamps)
            new_dm = dm.DataManager()
            new_dm.configure_directories()
            new_dm.create_new_run()
            time.sleep(1)
            new_dm.create_new_run()
            time.sleep(1)
            new_dm.create_new_run()
            # Assert there are now three run directories
            assert len(os.listdir(os.path.join(tmpdir, "runs"))) == 3
            # Create a file in the most recent run directory
            most_recent_run = sorted(os.listdir(os.path.join(tmpdir, "runs")))[-1]
            with open(os.path.join(tmpdir, "runs", most_recent_run, "filt", "test.txt"), "w") as file:
                file.write("not empty!")
            # Remove the empty run directories
            new_dm.remove_empty_run_directories()
            # Assert there's just one run directory
            assert len(os.listdir(os.path.join(tmpdir, "runs"))) == 1


def test_remove_empty_skips_nonempty_non_symlinked_run(unset_env_vars):
    """Make sure that the method to remove empty run directories does not remove
    non-empty directories. This is the case where the non-empty directory is not
    the symlinked work directory, so we need to generate a new run directory to
    preserve the symlink."""
    with tempfile.TemporaryDirectory() as tmpdir:
        with patch("lephare.data_manager.user_cache_dir", return_value=tmpdir):
            # Set up data manager and make three runs (sleeping for diff timestamps)
            new_dm = dm.DataManager()
            new_dm.configure_directories()
            new_dm.create_new_run()
            time.sleep(1)
            new_dm.create_new_run()
            time.sleep(1)
            new_dm.create_new_run()
            # Assert there are now three run directories
            assert len(os.listdir(os.path.join(tmpdir, "runs"))) == 3
            # Create a file in the first run directory
            first_run = sorted(os.listdir(os.path.join(tmpdir, "runs")))[0]
            with open(os.path.join(tmpdir, "runs", first_run, "filt", "test.txt"), "w") as file:
                file.write("not empty!")
            # Remove the empty run directories
            new_dm.remove_empty_run_directories()
            # Assert there's two run directories: the non-empty one and the new one
            assert len(os.listdir(os.path.join(tmpdir, "runs"))) == 2



import matplotlib

matplotlib.use("Agg")

import os

import lephare as lp

TESTDIR = os.path.abspath(os.path.dirname(__file__))
TESTDATADIR = os.path.join(TESTDIR, "../data")


def test_spec_plotspec():
    lp._spec.plotspec(os.path.join(TESTDATADIR, "example.spec"))
    assert True



import os

import pytest
from lephare import GalMag, MagSvc, QSOMag, StarMag


def test_magsvc(test_data_dir):
    # Test MagSvc class
    config_file_path = os.path.join(test_data_dir, "examples/COSMOS.para")
    keymap = MagSvc.from_config(objtype="GAL", config_file=config_file_path)
    assert type(keymap) == GalMag

    keymap = MagSvc.from_config(objtype="Star", config_file=config_file_path)
    assert type(keymap) == StarMag

    keymap = MagSvc.from_config(objtype="qso", config_file=config_file_path)
    assert type(keymap) == QSOMag


def test_magsvc_unknown_objtype(test_data_dir):
    # Test MagSvc class with unknown objtype
    config_file_path = os.path.join(test_data_dir, "examples/COSMOS.para")

    with pytest.raises(ValueError) as excinfo:
        _ = MagSvc.from_config(objtype="UNKNOWN", config_file=config_file_path)
        assert excinfo.value == "Unknown objtype: UNKNOWN"
